#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2012, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import pika

from stratuslab.messaging.MsgBase import MsgBase

class AmqpClient(MsgBase):
    def __init__(self, configHolder):
        super(AmqpClient, self).__init__(configHolder)
        
        self.connection = None
        self.connection_params = pika.ConnectionParameters(self.msg_endpoint)
        
    def connect(self):
        self.connection = pika.BlockingConnection(self.connection_params)
        self.channel = self.connection.channel()
        
    def send(self, message):
        self.channel.queue_declare(queue=self.msg_queue)
        self.channel.basic_publish(exchange='',
                                   routing_key=self.msg_queue,
                                   body=message)

    def disconnect(self):
        self.connection.close()



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os

import stratuslab.Util as Util
from Configurable import Configurable
from Exceptions import ConfigurationException

class Signator(Configurable):

    @staticmethod
    def findJar():
        dirs = []
        module_dirname = os.path.dirname(__file__)
        devRelativePath = '../../../../../../stratuslab-marketplace/metadata/target'
        dirs.append(os.path.join(module_dirname, devRelativePath))
        tarballRelativePath = '../../../../java/'
        dirs.append(os.path.join(module_dirname, tarballRelativePath))
        dirs.append('/var/lib/stratuslab/java')
        
        for dir in dirs:
            try:
                jarFile = Util.fileFind(dir, 'metadata', 'dependencies.jar')
                return jarFile
            except ValueError:
                pass

    def __init__(self, manifestFile, configHolder):
        self.outputManifestFile = None
        self.renamedInputManifestFile = manifestFile
        self.email = ''
        super(Signator, self).__init__(configHolder)
        self.manifestFile = manifestFile
        if not self.outputManifestFile:
            self.outputManifestFile = self.manifestFile
            self.renamedInputManifestFile = self.manifestFile + '.orig'
        self.tempManifestFile = self.manifestFile + '.new'

    def sign(self):
        res, output = self._sign()
        if res:
            Util.printError(output, exit=False)
            self._cleanupTempFile()
        else:
            self._renameFiles()
        return res

    def _sign(self):
        jarLocation = self._findJar()
        self.printDetail('Signature jar file: %s' % jarLocation)
        javaMainArgs = ' ' + self.manifestFile + ' ' + self.tempManifestFile + \
                       ' ' + self.p12Certificate + ' ' + self.p12Password + \
                       ' ' + self.email
        cmd = os.path.join('java -cp %s %s' % (jarLocation, 'eu.stratuslab.marketplace.metadata.SignMetadata'))
        cmd += javaMainArgs
        self._printCalling(cmd)
        return Util.execute(cmd.split(' '),withOutput=True)

    def _findJar(self):
        jarLocation = self.findJar()
        if not jarLocation:
            raise ConfigurationException('Failed to find metadata jar file')
        return jarLocation

    def _renameFiles(self):
        self.printDetail('Renaming input file from %s to %s' % (self.manifestFile, self.renamedInputManifestFile), verboseThreshold=Util.VERBOSE_LEVEL_DETAILED)
        os.rename(self.manifestFile, self.renamedInputManifestFile)
        self.printDetail('Renaming output file from %s to %s' % (self.tempManifestFile, self.outputManifestFile), verboseThreshold=Util.VERBOSE_LEVEL_DETAILED)
        os.rename(self.tempManifestFile, self.outputManifestFile)

    def _cleanupTempFile(self):
        if os.path.isfile(self.tempManifestFile):
            os.remove(self.tempManifestFile)

    def validate(self):
        jarLocation = self._findJar()
        javaMainArgs = ' ' + self.manifestFile
        cmd = os.path.join('java -cp %s %s' % (jarLocation, 'eu.stratuslab.marketplace.metadata.CheckMetadata'))
        cmd += javaMainArgs
        self._printCalling(cmd)
        return Util.execute(cmd.split(' '))

    def _printCalling(self, cmd):
        Util.printDetail('Calling: %s' % cmd, verboseLevel=self.verboseLevel,
                         verboseThreshold=Util.VERBOSE_LEVEL_DETAILED)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552.
#
# Copyright (c) 2012, Centre National de la Recherche Scientifique
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from stratuslab import Util

etree = Util.importETree()

class PortTranslation(object):

    def __init__(self, configHolder):
        self.verboseLevel = 0
        configHolder.assign(self)

    def isTranslatedNetwork(self, network):
        return Util.isTrueConfVal(self.portTranslation) and network in self.patNetworks

    def hasPortTranslated(self, vmInfo):
        if getattr(vmInfo, 'template_pat', None):
            return True
        return False

    def findGatewayPort(self, vmInfo, port):
        ports = getattr(vmInfo, 'template_pat', None)
        vmPorts = VmPortTranslation(ports)
        return vmPorts.findGwPortFromVmPort(port)

    def addPortTranslationToVmsInfo(self, vmsInfo):
        # Get all ports in one request to avoid too many access to
        # PortTranslation service.
        vmsPorts = self.getPortTranslation()
        for vmInfo in vmsInfo.findall('VM'):
            vmId = vmInfo.find('ID').text
            vmPorts = vmsPorts.get(vmId, VmPortTranslation(vmid=vmId))
            self.addPortTranslationToSingleVmInfo(vmInfo, vmPorts)

    def addPortTranslationToSingleVmInfo(self, vmInfo, vmPorts=None):
        vmId = vmInfo.find('ID').text
        vmTemplate = vmInfo.find('TEMPLATE')

        if vmPorts is None:
            vmPorts = self.getVmPortTranslation(vmId)

        if len(vmPorts):
            labelElement = etree.Element('PAT')
            labelElement.text = vmPorts.toString()
            vmTemplate.append(labelElement)
        # FIXME: should be a loop over each port translation but doesn't work
        #        because of a bug (JIRA#1036)
        #labelElement = etree.Element('PAT')
        #for vmPort in vmPorts:
        #    labelElement.text = self._formatPortTranslation(vmPort)
        #    vmTemplate.append(labelElement)

    def getAllPortTranslation(self):
        return {}

    def getVmPortTranslation(self, vmId):
        return VmPortTranslation(vmid=vmId)


class VmPortTranslation(object):

    def __init__(self, *args, **kwargs):
        self.vmPorts = []
        self.vmId = None

        if len(args) == 1 and isinstance(args[0], str):
            self.vmPorts = VmPortTranslation.fromString(args[0])
        else:
            for arg in args:
                if isinstance(arg, list) or isinstance(arg, tuple):
                    self.add(*arg)

        if kwargs.has_key('vmid'):
            self.vmId = kwargs['vmid']

    @staticmethod
    def fromDict(data):
        """ {vmId: [(vmPort, gwPort), ...], ...} => {vmId: [VmPortTranslation, ...], ...} """
        result = {}
        if isinstance(data, dict):
            for vmId, vmPorts in data.iteritems():
                result[vmId] = VmPortTranslation(*vmPorts, vmid=vmId)
        return result

    @staticmethod
    def fromTuple(tup):
        """ [(vmId, vmPort, gwPort), ...] ==> {vmId: [VmPortTranslation, ...], ...} """
        result = {}
        for vmId, vmPort, gwPort in tup:
            if not result.has_key(vmId):
                result[vmId] = VmPortTranslation(vmid=vmId)
            result[vmId].add(vmPort, gwPort)
        return result

    @staticmethod
    def fromString(string):
        return [tuple(ports.split(':')) for ports in string.split(',')]

    @staticmethod
    def fromStringWithVm(string):
        vmId, ports = string.split('=')
        return vmId, self.fromString(ports)

    def toString(self):
        return ','.join([':'.join(port) for port in self.vmPorts])

    def toStringWithVm(self):
        return '='.join((self.vmId, self.toString()))

    def toList(self):
        return self.vmPorts

    def toDict(self):
        return {str(self.vmId): self.toList()}

    def add(self, vmPort, gwPort):
        self.vmPorts.append((str(vmPort), str(gwPort)))

    def __len__(self):
        return len(self.vmPorts)

    def __str__(self):
        return str(self.vmPorts)

    def __repr__(self):
        return str(self.vmPorts)

    def findGwPortFromVmPort(self, port):
        for vmPort, gwPort in self.vmPorts:
            if vmPort == port:
                return gwPort
        return None

    def findVmPortFromGwPort(self, port):
        for vmPort, gwPort in self.vmPorts:
            if gwPort == port:
                return vmPort
        return None




#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, GRNET S.A.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import unittest

from stratuslab.Runner import Runner
from stratuslab.ConfigHolder import ConfigHolder
from stratuslab.Cluster import Cluster

class ClusterTest(unittest.TestCase):
    instanceNumber = 2
    image = "CcMFFcnWsQaFpq__Bk_NmP73faX"
    instanceType = "m1.xlarge"
    username = 'oneadmin'
    password = 'oneadmin'

    def testDeployNFSCluster(self):
        options = Runner.defaultRunOptions()
        options.update({'username': ClusterTest.username,
                        'password': ClusterTest.password,
                        'mpi_machine_file': True, 
                        'instanceType': self.instanceType,
                        'cluster_admin': 'root', 
                        'cluster_user':'testuser',
                        'master_vmid': None,
                        'tolerate_failures': False, 
                        'clean_after_failure': False,
                        'include_master': True, 
                        'shared_folder':'/home',
                        'add_packages': None, 
                        'ssh_hostbased': False, 
                        'instanceNumber': self.instanceNumber,
                        'verboseLevel':0, 
                        'marketplaceEndpoint':'https://marketplace.stratuslab.eu'})
        configHolder = ConfigHolder(options)
        runner = Runner(self.image, configHolder)
        cluster = Cluster(configHolder, runner, options['master_vmid'])
        runner.runInstance()

        self.assertTrue(cluster.deploy() in [0, 128])

        runner.killInstances(runner.vmIds)

    def testDeploySSHCluster(self):
        options = Runner.defaultRunOptions()
        options.update({'username': ClusterTest.username,
                        'password': ClusterTest.password,
                        'mpi_machine_file': True, 
                        'instanceType': self.instanceType,
                        'cluster_admin': 'root', 
                        'cluster_user':'testuser',
                        'master_vmid': None,
                        'tolerate_failures': False, 
                        'clean_after_failure': False,
                        'include_master': True, 
                        'shared_folder': None,
                        'add_packages': None, 
                        'ssh_hostbased': True, 
                        'instanceNumber': self.instanceNumber,
                        'verboseLevel':0, 
                        'marketplaceEndpoint':'https://marketplace.stratuslab.eu'})
        configHolder = ConfigHolder(options)
        runner = Runner(self.image, configHolder)
        cluster = Cluster(configHolder, runner, options['master_vmid'])
        runner.runInstance()
        
        self.assertTrue(cluster.deploy() in [0, 128])

        runner.killInstances(runner.vmIds)

    def testHeterogeneousCluster(self):
        # Master node instance
        options = Runner.defaultRunOptions()
        options.update({'username': ClusterTest.username,
                        'password': ClusterTest.password,
                        'instanceType': 'm1.large', 
                        'instanceNumber': 1, 
                        'verboseLevel':0,
                        'marketplaceEndpoint':'https://marketplace.stratuslab.eu'})
        configHolder = ConfigHolder(options)
        runner = Runner(self.image, configHolder)
        runner.runInstance()

        masterId = runner.vmIds

        # Worker node instance
        options.update({'username': ClusterTest.username,
                        'password': ClusterTest.password,
                        'mpi_machine_file': True, 
                        'instanceType': self.instanceType,
                        'cluster_admin': 'root', 
                        'cluster_user':'testuser',
                        'master_vmid': runner.vmIds[0],
                        'include_master': True, 
                        'shared_folder': '/home',
                        'tolerate_failures': False, 
                        'clean_after_failure': False,
                        'add_packages': None, 
                        'ssh_hostbased': False, 
                        'instanceNumber': self.instanceNumber-1,
                        'verboseLevel':0, 
                        'marketplaceEndpoint':'https://marketplace.stratuslab.eu'})
        configHolder = ConfigHolder(options)
        runner = Runner(self.image, configHolder)
        cluster = Cluster(configHolder, runner, options['master_vmid'])
        runner.runInstance()

        self.assertTrue(cluster.deploy() in [0, 128])

        runner.killInstances(masterId)
        runner.killInstances(runner.vmIds)

if __name__ == "__main__":
    unittest.main()




#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2012, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os

from smtplib import SMTP
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase

from stratuslab.messaging.MsgBase import MsgBase
from stratuslab.messaging.Defaults import MSG_SENDER_EMAIL
from stratuslab.messaging.Defaults import MSG_SMTP_HOST

class EmailClient(MsgBase):
    def __init__(self, configHolder):
        self.smtp_host = MSG_SMTP_HOST
        self.subject = ''
        self.recipient = ''
        self.sender = ''

        super(EmailClient, self).__init__(configHolder)
        
        self.subject = self.subject or self.msg_queue
        self.recipient = self.recipient or self.msg_endpoint
        self.sender = self.sender or MSG_SENDER_EMAIL

    def send(self, message, attachment=''):
        email = self._compose_email(message, attachment)

        self._send_email(email)
    
    def _compose_email(self, message, attachment):
        email = MIMEMultipart()
        email['Subject'] = self.subject
        email['From'] = self.sender
        email['To'] = self.recipient
        email.attach(MIMEText(message))

        if attachment:
            part = MIMEBase('application', 'octet-stream')
            part.set_payload(open(attachment, 'rb').read())
            part.add_header('Content-Disposition',
                            'attachment; filename="%s"' % 
                                os.path.basename(attachment))
            email.attach(part)

        return email

    def _send_email(self, email):
        smtp = SMTP(self.smtp_host)
        smtp.sendmail(self.sender,
                      self.recipient,
                      email.as_string())
        smtp.quit()



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, Centre National de la Recherche Scientifique (CNRS)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import re

from stratuslab import Defaults
import stratuslab.Util
from stratuslab.Exceptions import ValidationException

class Util(object):

    ENVVAR_ENDPOINT = 'STRATUSLAB_MARKETPLACE_ENDPOINT'

    OPTION_STRING = '--marketplace-endpoint'

    @staticmethod
    def addEndpointOption(parser):
        default = os.getenv(Util.ENVVAR_ENDPOINT,
                            Defaults.marketplaceEndpoint)
        parser.add_option(Util.OPTION_STRING,
                          dest='marketplaceEndpoint',
                          help='Marketplace endpoint (hostname or URL). Default %s or %s' % \
                              (Util.ENVVAR_ENDPOINT,
                               Defaults.marketplaceEndpoint),
                          default=default)

        
    @staticmethod
    def checkEndpointOption(options):
        if not options.marketplaceEndpoint:
            options.marketplaceEndpoint = os.getenv(Util.ENVVAR_ENDPOINT,
                                                    Defaults.marketplaceEndpoint)

        options.marketplaceEndpoint = re.sub(r"/*$", '', options.marketplaceEndpoint)

        return True

    @staticmethod
    def metadataUrl(endpoint, identifier):
        if identifier.startswith('http'):
            return identifier
        
        if not endpoint:
            raise ValidationException('Marketplace endpoint is not provided ' +\
                                      'when building metadata URL.')

        _endpoint = stratuslab.Util.sanitizeEndpoint(endpoint, 
                                                     Defaults.marketplaceProtocol,
                                                     Defaults.marketplacePort)
        return '%s/metadata/%s' % (_endpoint, identifier)

    @staticmethod
    def metadataCompleteUrl(endpoint, identifier, email, created):
        return '%s/metadata/%s/%s/%s' % (endpoint, identifier, email, created)

    @staticmethod
    def metadataEndpointUrl(endpoint):
        return '%s/metadata' % endpoint



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
from datetime import datetime
import time
import tempfile

from stratuslab.CloudConnectorFactory import CloudConnectorFactory
from stratuslab.Runner import Runner
from stratuslab.Util import sshCmd
from stratuslab.Util import sshCmdWithOutput
from stratuslab.Util import waitUntilPingOrTimeout
from stratuslab.Util import getHostnameFromUri
import stratuslab.Util as Util
from Exceptions import ValidationException
from Exceptions import ExecutionException
from Authn import AuthnFactory
from stratuslab.system.ubuntu import installCmd as aptInstallCmd
from stratuslab.system.ubuntu import updateCmd as aptUpdateCmd
from stratuslab.system.ubuntu import cleanPackageCacheCmd as aptCleanPackageCacheCmd
from stratuslab.system.centos import installCmd as yumInstallCmd
from stratuslab.system.centos import updateCmd as yumUpdateCmd
from stratuslab.system.centos import cleanPackageCacheCmd as yumCleanPackageCacheCmd
from stratuslab.image.Image import Image
from stratuslab.system import Systems
from stratuslab import Defaults
from stratuslab.marketplace.ManifestDownloader import ManifestDownloader
from stratuslab.Monitor import Monitor


class Creator(object):
    VM_START_TIMEOUT = 60 * 10
    VM_PING_TIMEOUT = 60 * 5

    excludeFromCreatedImageDefault = ['/tmp/*',
                                      '/etc/ssh/ssh_host_*',
                                      '/root/.ssh/{authorized_keys,known_hosts}']

    def __init__(self, image, configHolder):
        self.image = image
        self.configHolder = configHolder

        self.newImageGroupName = ''
        self.newInstalledSoftwareName = ''
        self.newInstalledSoftwareVersion = ''
        self.newImageGroupVersion = ''
        self.newImageGroupVersionWithManifestId = False
        self.author = ''
        self.title = ''
        self.comment = ''
        self.os = ''
        self.authorEmail = ''
        self.marketplaceEndpointNewimage = ''

        self.endpoint = ''

        self.extraOsReposUrls = ''
        self.packages = ''

        self.scripts = ''
        self.prerecipe = ''
        self.recipe = ''

        self.verboseLevel = ''

        self.shutdownVm = True

        self.signManifest = True

        self.vmStartTimeout = self.VM_START_TIMEOUT
        self.vmPingTimeout = self.VM_PING_TIMEOUT

        self.options = Runner.defaultRunOptions()
        self.options.update(configHolder.options)
        self.configHolder.options.update(self.options)

        configHolder.assign(self)

        dateNow = datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
        self.stdout = open('/tmp/stratuslab_%s.log' % dateNow, 'a')
        self.stderr = open('/tmp/stratuslab_%s.err' % dateNow, 'a')

        credentials = AuthnFactory.getCredentials(self)
        self.cloud = CloudConnectorFactory.getCloud(credentials)
        self.cloud.setEndpoint(self.endpoint)

        self.runner = None
        self.vmAddress = None
        self.vmId = None
        self.vmIp = None
        self.vmName = 'creator'

        self.userPublicKeyFile = self.options.get('userPublicKeyFile',
                                                  Defaults.sshPublicKeyLocation)
        self.userPrivateKeyFile = self.userPublicKeyFile.strip('.pub')

        self.mainDisk = ''
        self.extraDisk = ''
        self.mountPointExtraDisk = '/media'
        self.imageFile = ''
        self.imageFileBundled = ''

        self.excludeFromCreatedImage = \
            self.excludeFromCreatedImageDefault + \
            self.options.get('excludeFromCreatedImage', '').split(',')

        self.installer = self.options.get('installer')

        self.targetImageUri = ''
        self.targetManifestUri = ''

        self.manifest = ''
        self.manifestObject = None
        self.newManifestFileName = None

        self.manifestLocalFileName = ''

        self.__listener = CreatorBaseListener()

    def printDetail(self, msg):
        return Util.printDetail(msg, self.verboseLevel, Util.VERBOSE_LEVEL_NORMAL)

    def create(self):

        self._printAction('Starting image creation')

        self.startNode()
        try:
            self.buildNodeIncrement()
            self._printAction('Finished building image increment.')
            self._printAction('Please check %s for new image ID and instruction.' %
                              self.authorEmail)
        finally:
            self._shutdownNode()
        self._localCleanUp()

    def startNode(self):
        self._imageExists()

        self._retrieveManifest()
        self.__setAttributesFromManifest()

        self.__createRunner()

        self._startMachine()

        self._waitMachineNetworkUpOrAbort()

        self._checkIfCanConnectToMachine()

    def buildNodeIncrement(self):

        self._executePrerecipe()
        self._installPackages()
        self._executeRecipe()
        self._executeScripts()

    def _printAction(self, msg):
        Util.printAction(msg)
        self._notifyOnAction(msg)

    def _printStep(self, msg):
        Util.printStep(msg)
        self._notifyOnStep(msg)

    def _printError(self, msg):
        self._notifyOnError(msg)
        Util.printError(msg)

    def setListener(self, listener):
        if listener:
            self.__listener = listener

    def _notifyOnAction(self, note):
        self._notify('Action', note)

    def _notifyOnStep(self, note):
        self._notify('Step', note)

    def _notifyOnError(self, note):
        self._notify('Error', note)

    def _notify(self, operation, note):
        def callListener():
            notifyFunction = getattr(self.__listener, onOperation)
            notifyFunction(note)

        onOperation = 'on%s' % operation

        if hasattr(self.__listener, onOperation):
            pass
        elif hasattr(self.__listener, 'onAny'):
            onOperation = 'onAny'
        callListener()

    def _checkIfCanConnectToMachine(self):
        self._printStep('Check if we can connect to the machine')

        cmd = 'true'
        try:
            self._sshCmdWithOutputVerb(cmd)
        except ExecutionException:
            sleepTime = 6
            maxCount = 40
            counter = 0
            while True:
                try:
                    self.printDetail('Sleeping %i sec. Retry %i out of %i.' % (sleepTime, counter + 1, maxCount))
                    time.sleep(sleepTime)
                    self._sshCmdWithOutputVerb(cmd)
                    break
                except ExecutionException, e:
                    if counter >= maxCount:
                        raise ExecutionException(e)
                    counter += 1

    def _imageExists(self):
        self._printStep('Checking that base image exists')
        self._checkImageExists()

    def _checkImageExists(self):
        image = Image(self.configHolder)
        image.checkImageExists(self.image)

    def _getCreateImageTemplateDict(self):
        return {Runner.CREATE_IMAGE_KEY_CREATOR_EMAIL: self.authorEmail,
                Runner.CREATE_IMAGE_KEY_CREATOR_NAME: self.author,
                Runner.CREATE_IMAGE_KEY_NEWIMAGE_TITLE: self.title,
                Runner.CREATE_IMAGE_KEY_NEWIMAGE_COMMENT: self.comment,
                Runner.CREATE_IMAGE_KEY_NEWIMAGE_VERSION: self.newImageGroupVersion,
                Runner.CREATE_IMAGE_KEY_NEWIMAGE_MARKETPLACE: self.marketplaceEndpointNewimage}

    def createRunner(self):
        self.__createRunner()

    def __createRunner(self):
        self.configHolder.set('vmName',
                              '%s: %s' % (self.vmName, Util.getTimeInIso8601()))
        self.configHolder.set('noCheckImageUrl', True)

        self.configHolder.set('saveDisk', True)

        self.runner = Runner(self.image, self.configHolder)

        self.runner.updateCreateImageTemplateData(
            self._getCreateImageTemplateDict())

    def _startMachine(self):
        self._printStep('Starting base image')

        try:
            self.vmId = self.runner.runInstance()[0]
        except Exception, msg:
            self._printError('An error occurred while starting machine: \n\t%s' % msg)
        try:
            _, self.vmIp = self.runner.getNetworkDetail(self.vmId)
            self.vmAddress = self.vmIp
        except Exception, e:
            self._printError('An error occurred while getting machine network details: \n\t%s' % str(e))

        self._printStep('Waiting for machine to boot')
        vmStarted = self.runner.waitUntilVmRunningOrTimeout(self.vmId,
                                                            self.vmStartTimeout,
                                                            failOn='Failed')
        if not vmStarted:
            if self.runner.getVmState(self.vmId) == 'Failed':
                msg = 'Failed to start VM (id=%s, ip=%s): %s' % \
                      (self.vmId, self.vmAddress,
                       self._getVmFailureMessage(self.vmId))
            else:
                msg = 'Failed to start VM within %i seconds (id=%s, ip=%s)' % \
                      (self.vmStartTimeout, self.vmId, self.vmAddress)
            self.printDetail(msg)
            self._killMachine()
            self._printError(msg)

    def _stopMachine(self):
        self._printStep('Shutting down machine')

        if self.getVmState() != 'Failed':
            self.cloud.vmStop(self.vmId)

    def _killMachine(self):
        self._printStep('Killing machine')

        if self.vmId:
            self.cloud.vmKill(self.vmId)
        else:
            Util.printWarning('Undefined VM ID, when trying to kill machine.')

    def _getVmFailureMessage(self, vmId):
        return getattr(Monitor(self.configHolder)._vmDetail(vmId),
                       'template_error_message', '')

    def _shutdownNode(self):
        if self.shutdownVm:
            self._stopMachine()
        else:
            self._printStep('Machine ready for use')
            msg = '\n\tMachine IP: %s\tRemember to stop the machine when finished' % self.vmIp
            Util.printInfo(msg)

    def _waitMachineNetworkUpOrAbort(self):
        self._printStep('Waiting for machine network to start')
        if not waitUntilPingOrTimeout(self.vmAddress, self.vmPingTimeout):
            msg = 'Unable to ping VM in %i seconds (id=%s, ip=%s)' % \
                  (self.vmPingTimeout, self.vmId, self.vmAddress)
            self._printError(msg)
            self._stopMachine()

    def _getPublicAddress(self):
        return self.vmIp

    def _retrieveManifest(self):
        self._printStep('Retrieving image manifest')

        configHolder = self.configHolder.copy()

        downloader = ManifestDownloader(configHolder)
        self.manifestObject = downloader.getManifestInfo(self.image)
        self.manifest = self.manifestObject.tostring()

    def __setAttributesFromManifest(self):
        self._setOsFromManifest()
        self._setInstallerBasedOnOs()

    def _setOsFromManifest(self):
        if not self.os:
            self.os = self._getAttrFromManifest('os').lower()

    def _setInstallerBasedOnOs(self):
        if not self.installer:
            self.installer = Systems.getInstallerBasedOnOs(self.os)

    def _getAttrFromManifest(self, attr):
        return getattr(self.manifestObject, attr)

    def _installPackages(self):
        self._printStep('Installing user packages')

        if len(self.packages) == 0:
            self.printDetail('No packages to install')
            return

        self._setUpExtraRepositories()

        self.printDetail('Updating installer')

        ret = self._doInstallerUpdate()

        self.printDetail('Installing packages: %s' % self.packages)

        ret = self._doInstallPackagesRemotly(self.packages)

        if ret != 0:
            self._printError('An error occurred while installing packages')

    def _setUpExtraRepositories(self):
        if not self.extraOsReposUrls:
            return

        self.printDetail('Adding extra repositories')

        if self.installer not in Systems.INSTALLERS:
            ValidationException('Unknown installer %s. Bailing out.' %
                                self.installer)

        extraReposList = self.extraOsReposUrls.split(',')

        if self.installer == 'yum':
            for i, repoUrl in enumerate(extraReposList):
                repoName = getHostnameFromUri(repoUrl)
                cmd = """cat >> /etc/yum.repos.d/%(name)s.repo << EOF
[%(name)s]
name=%(name)s
baseurl=%(url)s
gpgcheck=0
enabled=1
EOF
""" % {'name': '%s-%i' % (repoName, i), 'id': i, 'url': repoUrl}
        elif self.installer == 'apt':
            for repoUrl in extraReposList:
                repoName = getHostnameFromUri(repoUrl)
                cmd = """cat >> /etc/apt/sources.list.d/%(reponame)s.list  << EOF
deb %(repourl)s
EOF
""" % {'reponame': repoName, 'repourl': repoUrl}

        self._sshCmdWithOutput(cmd)

    def _doInstallPackagesRemotly(self, packages):
        cmd = self._buildInstallerCommand() + ' '
        cmd += ' '.join(packages.split(','))
        return self._sshCmd(cmd, stderr=self.stderr, stdout=self.stdout)

    def _doInstallerUpdate(self):
        cmd = self._buildUpdaterCommand()
        return self._sshCmd(cmd, stderr=self.stderr, stdout=self.stdout)

    def _buildInstallerCommand(self):
        if self.installer == 'yum':
            return yumInstallCmd
        elif self.installer == 'apt':
            return aptInstallCmd

    def _buildUpdaterCommand(self):
        if self.installer == 'yum':
            return yumUpdateCmd
        elif self.installer == 'apt':
            return aptUpdateCmd

    def _buildPackageCacheCleanerCommand(self):
        if self.installer == 'yum':
            return yumCleanPackageCacheCmd
        elif self.installer == 'apt':
            return aptCleanPackageCacheCmd

    def _executeScripts(self):
        self._printStep('Executing user scripts')

        if len(self.scripts) == 0:
            self.printDetail('No scripts to execute')
            return

        self.printDetail('Executing scripts: %s' % self.scripts)

        for script in self.scripts.split(','):
            self._uploadAndExecuteRemoteScript(script)

    def _uploadAndExecuteRemoteScript(self, script):

        def __tellScriptNameAndArgs(script):
            scriptNameAndArgs = os.path.basename(script)
            scriptNameAndArgsList = scriptNameAndArgs.split(' ', 1)
            if len(scriptNameAndArgsList) == 1:  # no arguments given
                scriptNameAndArgsList = scriptNameAndArgsList + ['']
            return scriptNameAndArgsList

        def _uploadScript(script):

            scriptName, args = __tellScriptNameAndArgs(script)

            scriptDirectory = Util.sanitizePath(os.path.dirname(script))
            scriptPathLocal = os.path.abspath(os.path.join(scriptDirectory, scriptName))
            scriptPathRemote = '/tmp/%s' % scriptName

            rc, output = self._scpWithOutput(scriptPathLocal, 'root@%s:%s' % (self.vmAddress, scriptPathRemote))
            if rc != 0:
                self._printError('An error occurred while uploading script %s\n%s' % (script, output))

            self._sshCmdWithOutput('chmod 0755 %s' % scriptPathRemote)

            return scriptPathRemote, args

        def _executeRemoteScript(scriptPathRemote, args=''):
            rc = self._sshCmd('%s %s' % (scriptPathRemote, args), throwOnError=False,
                              pseudoTTY=True)
            if rc != 0:
                self._printError('An error occurred while executing script %s' % script)

        scriptPathRemote, args = _uploadScript(script)
        _executeRemoteScript(scriptPathRemote, args)

    def _executePrerecipe(self):
        self._printStep('Executing user prerecipe')

        if len(self.prerecipe) == 0:
            self.printDetail('No prerecipe to execute')
            return

        self._uploadAndExecuteRemoteRecipe(self.prerecipe)

    def _executeRecipe(self):
        self._printStep('Executing user recipe')

        if len(self.recipe) == 0:
            self.printDetail('No recipe to execute')
            return

        self._uploadAndExecuteRemoteRecipe(self.recipe)

    def _uploadAndExecuteRemoteRecipe(self, script):

        fd, recipeFile = tempfile.mkstemp()
        try:
            os.write(fd, script)
            os.close(fd)
            os.chmod(recipeFile, 0755)
            scriptPath = '/tmp/%s' % os.path.basename(recipeFile)
            rc = self._scp(recipeFile, 'root@%s:%s' % (self.vmAddress, scriptPath))
            if rc != 0:
                self._printError('An error occurred while uploading recipe')
            self._sshCmdWithOutput('chmod 0755 %s' % scriptPath)

            rc = self._sshCmd(scriptPath, throwOnError=False, pseudoTTY=True)
            if rc != 0:
                self._printError('An error occurred while executing user recipe.')
        finally:
            try:
                os.unlink(recipeFile)
            except:
                pass

    def _localCleanUp(self):
        Util.execute(['rm', '-rf', self.manifestLocalFileName])

    def _scp(self, src, dst, **kwargs):
        return Util.scp(src, dst, self.userPrivateKeyFile,
                        verboseLevel=self.verboseLevel, verboseThreshold=Util.VERBOSE_LEVEL_DETAILED,
                        stderr=self.stderr, stdout=self.stdout, **kwargs)

    def _scpWithOutput(self, src, dst):
        return self._scp(src, dst, withOutput=True)

    def _sshCmd(self, cmd, throwOnError=True, **kwargs):
        ret = sshCmd(cmd, self.vmAddress,
                     sshKey=self.userPrivateKeyFile,
                     verboseLevel=self.verboseLevel,
                     verboseThreshold=Util.VERBOSE_LEVEL_DETAILED,
                     **kwargs)
        if ret and throwOnError:
            raise ExecutionException('Error executing command: %s' % cmd)
        return ret

    def _sshCmdWithOutput(self, cmd, throwOnError=True, **kwargs):
        rc, output = sshCmdWithOutput(cmd, self.vmAddress,
                                      sshKey=self.userPrivateKeyFile,
                                      verboseLevel=self.verboseLevel,
                                      verboseThreshold=Util.VERBOSE_LEVEL_DETAILED,
                                      **kwargs)
        if rc and throwOnError:
            raise ExecutionException('Error executing command: %s\n%s' % (cmd, output))
        return rc, output

    def _sshCmdWithOutputVerb(self, cmd, **kwargs):
        return self._sshCmdWithOutput(cmd, sshVerb=True, **kwargs)

    def _sshCmdWithOutputQuiet(self, cmd, **kwargs):
        return self._sshCmdWithOutput(cmd, sshQuiet=True, **kwargs)

    def getNewImageId(self):
        return self.manifestObject.identifier

    def getVmId(self):
        return self.vmId

    def getVmState(self):
        return self.runner.getVmState(self.vmId)


# FIXME: This should be treated as a log handler rather than an ad hoc class.
class CreatorBaseListener(object):
    def __init__(self, verbose=False):
        if verbose:
            self.write = self.__beVerbose

    def write(self, msg):
        pass

    def __beVerbose(self, msg):
        print msg

    def onAction(self, msg):
        self.write('action: %s' % msg)

    def onStep(self, msg):
        self.write('step: %s' % msg)

    def onError(self, msg):
        self.write('error: %s' % msg)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import sys

from Exceptions import ValidationException

class VersionChecker(object):

    MINIMUM_VERSION = (2, 6)

    def _getPythonVersion(self):
        return sys.version_info
    
    def check(self):
        if self._getPythonVersion() < VersionChecker.MINIMUM_VERSION:
            raise ValidationException('python version must be %s or greater' % '.'.join(map(str,VersionChecker.MINIMUM_VERSION)))



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import shutil

class FileAppender(object):
    
    def __init__(self, filename):
        self.filename = filename
        self.lines = []
        self.newLines = []
        self.foundExit = False
    
    def insertAtTheEnd(self, newLine):
        ''' Append at the end of the file (e.g. rc.local) the newLine
            at the end of the file, unless the file ends with 'exit',
            in which case it inserts just before.'''
        
        self._backupIfNecessary()
        
        self._reverseLines()

        newLine = newLine + '\n' 
        
        self.foundExit = False
        for line in self.lines:
            if line.strip() == '':
                self._appendNewLine(line)
                continue
            if self._containsExit(line):
                self._insertLines(line, newLine)
                continue
            if self.foundExit:
                self._appendNewLine(line)
                continue
            self._insertLines(newLine, line)
        
        self.newLines.reverse()
        
        self._writeAndClose()
    
    def _backupIfNecessary(self):
        originalFilename = self.filename + '.orig'
        if not os.path.exists(originalFilename):
            shutil.copyfile(self.filename, originalFilename)
    
    def _reverseLines(self):
        file = open(self.filename)
        self.lines = file.readlines()
        self.lines.reverse()
    
    def _appendNewLine(self, line):
        self.newLines.append(line)
    
    def _containsExit(self, line):
        return line.strip().startswith('exit')
    
    def _insertLines(self, first, second):
        self.foundExit = True
        self._appendNewLine(first)
        self._appendNewLine(second)
    
    def _writeAndClose(self):
        newfile = open(self.filename, 'w')
        newfile.writelines(self.newLines)
        newfile.close()
        os.chmod(self.filename, 0755)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552.
#
# Copyright (c) 2012, Centre National de la Recherche Scientifique
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import sys

from stratuslab import Util
from stratuslab.CommandBase import CommandBaseUser

class PortTranslationCommand(CommandBaseUser):

    @staticmethod
    def addCommonOptions(parser):
        parser.add_option('--pat-enable', dest='portTranslation',
                help='show port address translations',
                action='store_true', default=False)

        parser.add_option('--pat-service-host', dest='patServiceHost',
                help='set the hostname/ip of port translation service',
                action='store', metavar='HOST/IP')

        parser.add_option('--pat-gateway-host', dest='patGatewayHost',
                help='set the hostname/ip of port translation gateway',
                action='store', metavar='HOST/IP')

    def checkCommonOptions(self):
        self.checkServiceHost()
        self.checkGatewayHost()

    def checkServiceHost(self):
        self._setOptionIfNotDefined('patServiceHost', self.options.endpoint)

    def checkGatewayHost(self):
        self._setOptionIfNotDefined('patGatewayHost', self.options.endpoint)

    def _setOptionIfNotDefined(self, option, default):
        if not getattr(self.options, option, None):
            if self.verboseLevel >= Util.VERBOSE_LEVEL_NORMAL:
                sys.stdout.write("Warning: '%s' is not defined, using '%s'.\n" % (option, default))
            setattr(self.options, option, default)




#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
from optparse import OptionParser

from stratuslab import Defaults
from stratuslab.AuthnCommand import UsernamePassword
from stratuslab.CommandBase import CommandBaseSysadmin


class StorageCommand(CommandBaseSysadmin):
    @staticmethod
    def pdiskOptions():
        return PDiskEndpoint.options()

    @staticmethod
    def addPDiskEndpointOptions(parser, defaultOptions=None):
        PDiskEndpoint.addOptions(parser, defaultOptions)

    @staticmethod
    def addVolumeOptions(parser):
        PDiskVolume.addOptions(parser)

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        StorageCommand.addPDiskEndpointOptions(parser, defaultOptions)
        StorageCommand.addVolumeOptions(parser)

    def checkPDiskEndpointOptionsOnly(self):
        if not self.checkPDiskEndpointOptions():
            self.parser.error('Missing persistent disk endpoint. Please provide %s'
                              % PDiskEndpoint.optionString)

    def checkPDiskEndpointOptions(self):
        return PDiskEndpoint.checkOptions(self.options)

    def checkVolumeOptions(self):
        PDiskVolume.checkOptions(self.options)

    def extractVolumeOptionsAsDict(self):
        return PDiskVolume.extractVolumeOptionsAsDict(self.options)


class PDiskEndpoint(object):
    optionString = '--pdisk-endpoint'

    @staticmethod
    def options():
        return {'pdiskEndpoint': os.getenv('STRATUSLAB_PDISK_ENDPOINT', ''),
                'pdiskProtocol': os.getenv('STRATUSLAB_PDISK_PROTOCOL', Defaults.pdiskProtocol),
                'pdiskPort': os.getenv('STRATUSLAB_PDISK_PORT', Defaults.pdiskPort),
                'pdiskUsername': os.getenv('STRATUSLAB_PDISK_USERNAME', UsernamePassword.options().get('username')),
                'pdiskPassword': os.getenv('STRATUSLAB_PDISK_PASSWORD', UsernamePassword.options().get('password'))}

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        if not defaultOptions:
            defaultOptions = PDiskEndpoint.options()

        # TODO: Add certificate support
        parser.add_option('--pdisk-endpoint', dest='pdiskEndpoint',
                          help='Persistent Disk service endpoint (hostname or URL). Default STRATUSLAB_PDISK_ENDPOINT',
                          default=defaultOptions['pdiskEndpoint'])
        parser.add_option('--pdisk-username', dest='pdiskUsername',
                          help='Persistent Disk service username. \
                          Default STRATUSLAB_PDISK_USERNAME, then your cloud username',
                          metavar='NAME', default=defaultOptions['pdiskUsername'])
        parser.add_option('--pdisk-password', dest='pdiskPassword',
                          help='Persistent Disk service password. \
                          Default STRATUSLAB_PDISK_PASSWORD, then your cloud password',
                          metavar='NAME', default=defaultOptions['pdiskPassword'])

    @staticmethod
    def checkOptions(options):
        if options.pdiskEndpoint:
            return True
        return False

    @staticmethod
    def checkOptionsRaiseOnError(options):
        parser = OptionParser(version="13.05.0.RC1")
        if not options.pdiskEndpoint:
            parser.error('Missing Persistent Disk service endpoint. Please provide %s'
                         % PDiskEndpoint.optionString)


class PDiskVolume(object):
    TAG_LENGTH_MAX = 40
    TAG_DEFAULT = None

    @staticmethod
    def addOptions(parser):
        parser.add_option('-t', '--tag', dest='volumeTag',
                          help='Tag of the volume.',
                          default=PDiskVolume.TAG_DEFAULT)

        parser.add_option('--private', dest='volumeVisibility',
                          help='''Set to private image''',
                          action='store_const', const="PRIVATE")

        parser.add_option('--public', dest='volumeVisibility',
                          help='''Set to public image''',
                          action='store_const', const="PUBLIC")

        parser.add_option('--machine-image-origin', dest='volumeType',
                          help='''Flag as original machine image''',
                          action='store_const', const="MACHINE_IMAGE_ORIGIN")

        parser.add_option('--machine-image-live', dest='volumeType',
                          help='''Flag as live machine image''',
                          action='store_const', const="MACHINE_IMAGE_LIVE")

        parser.add_option('--data-image-origin', dest='volumeType',
                          help='''Flag as original data image''',
                          action='store_const', const="DATA_IMAGE_ORIGIN")

        parser.add_option('--data-image-live', dest='volumeType',
                          help='''Flag as live data image''',
                          action='store_const', const="DATA_IMAGE_LIVE")

        parser.add_option('--data-image-raw-readonly', dest='volumeType',
                          help='''Flag as raw read-only data image''',
                          action='store_const', const="DATA_IMAGE_RAW_READONLY")

        parser.add_option('--data-image-raw-read-write', dest='volumeType',
                          help='''Flag as raw read-write data image''',
                          action='store_const', const="DATA_IMAGE_RAW_READ_WRITE")

    @staticmethod
    def checkOptions(options):
        PDiskVolume.checkTagLength(options)

    @staticmethod
    def checkTagLength(options):
        if not (options.volumeTag is PDiskVolume.TAG_DEFAULT):
            if len(options.volumeTag) > PDiskVolume.TAG_LENGTH_MAX:
                OptionParser(version="13.05.0.RC1"). \
                    error('Tags must have less than %d characters' %
                          PDiskVolume.TAG_LENGTH_MAX)

    @staticmethod
    def extractVolumeOptionsAsDict(options):
        keyvalues = {}

        if not (options.volumeTag is PDiskVolume.TAG_DEFAULT):
            keyvalues['tag'] = options.volumeTag

        if not (options.volumeVisibility is None):
            keyvalues['visibility'] = options.volumeVisibility

        if not (options.volumeType is None):
            keyvalues['type'] = options.volumeType

        return keyvalues



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import re
import urllib2

import stratuslab.Util as Util
from stratuslab.Util import printStep, printAction, printWarning, printError
import stratuslab.Exceptions as Exceptions
import stratuslab.cloudinit.Util as CloudInitUtil
from stratuslab.CloudConnectorFactory import CloudConnectorFactory
from stratuslab.Authn import AuthnFactory
from stratuslab.image.Image import Image
from stratuslab import Defaults
from stratuslab.AuthnCommand import CloudEndpoint
from stratuslab.commandbase.StorageCommand import PDiskEndpoint
from stratuslab.PersistentDisk import PersistentDisk
from marketplace.Util import Util as MarketplaceUtil
from stratuslab.ManifestInfo import ManifestInfo


class Runner(object):
    class HeadRequest(urllib2.Request):
        def get_method(self):
            return "HEAD"

    EXTRA_DISK = '''DISK=[
  FORMAT=ext3,
  READONLY=no,
  SAVE=no,
  SIZE=%(extraDiskSize)s,
  TARGET=%(vm_disks_prefix)sc,
  TYPE=fs ]'''

    # Don't hard code disk target to allow multiple pdisk attachment
    PERSISTENT_DISK = '''DISK=[
  SOURCE=pdisk:%(pdiskEndpointHostname)s:%(pdiskPort)s:%(persistentDiskUUID)s,
  TARGET=%(vm_disks_prefix)sc,
  TYPE=block ]'''

    # NOTE: The READONLY flag must NOT be set to true.  This will result
    # in a deployment.0 file that libvirt cannot deploy.  The device type
    # must also be kept with 'disk' and NOT 'cdrom', otherwise the
    # contextualization will not work correctly.  GitHub Issue #50.
    READONLY_DISK = '''DISK=[
  SOURCE="%(readonlyDiskId)s",
  READONLY=no,
  SAVE=no,
  TARGET=%(vm_disks_prefix)sc,
  DRIVER="raw" ]'''

    NOTIFICATION = '''NOTIFICATION = [
  HOST="{0}",
  VHOST="{1}",
  USER="{2}",
  PASSWORD="{3}",
  QUEUE="{4}" ]'''

    CREATE_IMAGE = '''CREATE_IMAGE = [
%s
]'''

    CREATE_IMAGE_KEY_CREATOR_EMAIL = 'CREATOR_EMAIL'
    CREATE_IMAGE_KEY_CREATOR_NAME = 'CREATOR_NAME'
    CREATE_IMAGE_KEY_NEWIMAGE_TITLE = 'NEWIMAGE_TITLE'
    CREATE_IMAGE_KEY_NEWIMAGE_COMMENT = 'NEWIMAGE_COMMENT'
    CREATE_IMAGE_KEY_NEWIMAGE_VERSION = 'NEWIMAGE_VERSION'
    CREATE_IMAGE_KEY_NEWIMAGE_MARKETPLACE = 'NEWIMAGE_MARKETPLACE'
    CREATE_IMAGE_KEY_MSG_TYPE = 'MSG_TYPE'
    CREATE_IMAGE_KEY_MSG_ENDPOINT = 'MSG_ENDPOINT'
    CREATE_IMAGE_KEY_MSG_QUEUE = 'MSG_QUEUE'
    CREATE_IMAGE_KEY_MSG_MESSAGE = 'MSG_MESSAGE'

    DISKS_BUS_AVAILABLE = ['ide', 'scsi', 'virtio']
    DISKS_BUS_DEFAULT = ManifestInfo.DISKS_BUS_DEFAULT
    DISKS_BUS_PREFIX_MAP = {'ide': 'hd',
                            'scsi': 'sd',
                            'virtio': 'vd'}
    vmDisksBus = None

    DEFAULT_INSTANCE_TYPE = 'm1.small'

    def __init__(self, image, configHolder):
        if image == '':
            raise ValueError('Image ID should be provided.')

        self.vm_image = image
        self.persistentDiskUUID = None
        self.readonlyDiskId = None
        self.extraDiskSize = None
        self.instanceNumber = 1
        self.authorEmail = ''
        self.marketplaceEndpointNewimage = ''
        self.pdiskEndpoint = None
        self.endpoint = None
        self.instanceType = Runner.DEFAULT_INSTANCE_TYPE
        self.vmCpu = ''
        self.vmRam = ''
        self.vmSwap = ''
        self.vmCpuAmount = ''
        self.vmName = ''
        self.vmKernel = ''
        self.vmRamdisk = ''
        self.isLocalIp = False
        self.isPrivateIp = False
        self.specificAddressRequest = False
        self.rawData = ''
        self.extraContextFile = ''
        self.extraContextData = ''
        self.cloudInit = ''
        self.vncPort = ''
        self.vncListen = ''
        self.noCheckImageUrl = False
        self.saveDisk = False
        self.userDefinedInstanceTypes = {}
        self.pdiskPort = Defaults.pdiskPort
        self.inboundPorts = None

        configHolder.assign(self)
        self.configHolder = configHolder

        self._setCloudContext()

        self.createImageData = {self.CREATE_IMAGE_KEY_CREATOR_EMAIL: self.authorEmail,
                                self.CREATE_IMAGE_KEY_NEWIMAGE_MARKETPLACE: self.marketplaceEndpointNewimage}

        self._initVmAttributes()

        self.instancesDetail = []

        self.availableInstanceTypes = self._getAvailableInstanceTypes()

        self._validateInstanceType()

    def _setCloudContext(self):
        credentials = AuthnFactory.getCredentials(self)
        self.cloud = CloudConnectorFactory.getCloud(credentials)

        if not self.pdiskEndpoint:
            self.pdiskEndpoint = self.endpoint

        self.endpoint = self.cloud.setEndpoint(self.endpoint)
        self.pdisk = None

    def _getAvailableInstanceTypes(self):
        availableTypes = Runner.getDefaultInstanceTypes()
        userDefinedTypes = self.userDefinedInstanceTypes

        availableTypes.update(userDefinedTypes)

        return availableTypes

    def _validateInstanceType(self):
        if self.instanceType not in self.availableInstanceTypes:
            raise ValueError('Unknown instance type: %s' % self.instanceType)

    def _initVmAttributes(self):
        if not self.vm_image:
            return

        self._initVmAttributesStatic()

        self._setMsgRecipients()
        self._setUserKeyIfDefined()
        self._setSaveDisk()
        self._setDiskImageFormat()
        self._setDisksBusType()
        self._setInboundPorts()

        # should go after all runtime fields are initialized
        self._setExtraDiskOptional()
        self._setPersistentDiskOptional()
        self._setReadonlyDiskOptional()

    def _initVmAttributesStatic(self):
        # VM template parameters initialization
        self.vm_cpu = 0
        self.vm_vcpu = 0
        self.vm_ram = 0
        self.vm_swap = 0
        self.vm_nic = ''
        self.vm_name = ''
        self.vm_disks_prefix = Runner.DISKS_BUS_PREFIX_MAP[Runner.DISKS_BUS_DEFAULT]
        self.vm_requirements = ''
        self.os_options = ''
        self.raw_data = ''
        self.extra_context = ''
        self.graphics = ''
        self.vmIds = []
        self.vmIdsAndNetwork = []
        self.diskImageFormat = None
        self.disk_driver = None
        self.inbound_ports = ''

    def _setMsgRecipients(self):
        try:
            self.msgRecipients
        except AttributeError:
            self.msgRecipients = []

    def _setDiskImageFormat(self):
        useQcowDiskFormat = getattr(self, 'useQcowDiskFormat', False)
        # if image ID was provided extract disk driver type from manifest
        if self.vm_image:
            if not useQcowDiskFormat and Image.isImageId(self.vm_image):
                image = Image(self.configHolder)
                self.disk_driver = image.getImageFormatByImageId(self.vm_image)
                return
        self.disk_driver = (useQcowDiskFormat and 'qcow2') or 'raw'

    def _setDisksBusType(self):
        if self.vmDisksBus:
            disks_bus = self.vmDisksBus
        else:
            if Image.isImageId(self.vm_image):
                image = Image(self.configHolder)
                disks_bus = image.getImageDisksBusTypeByImageId(self.vm_image)
            else:
                return

        try:
            self.vm_disks_prefix = Runner.DISKS_BUS_PREFIX_MAP[disks_bus]
        except KeyError:
            raise Exception('Unknown disks bus type %s' % disks_bus)

    def _setUserKeyIfDefined(self):
        if getattr(self, 'userPublicKeyFile', None):
            self.public_key = ''
            for line in file(self.userPublicKeyFile):
                if not line.startswith('Comment:'):
                    self.public_key += line

    def _setSaveDisk(self):
        self.save_disk = self.saveDisk and 'yes' or 'no'

    def _setExtraDiskOptional(self):
        try:
            self.extra_disk = (self.extraDiskSize and Runner.EXTRA_DISK % self.__dict__) or ''
        except AttributeError:
            pass

    def _setPersistentDiskOptional(self):
        if not self.persistentDiskUUID:
            return

        self.pdiskEndpointHostname = PersistentDisk.getFQNHostname(self.pdiskEndpoint)
        self.persistent_disk = (self.persistentDiskUUID and Runner.PERSISTENT_DISK % self.__dict__) or ''

        self._checkPersistentDiskAvailable()

    def _checkPersistentDiskAvailable(self):
        self.pdisk = PersistentDisk(self.configHolder)
        try:
            available, _ = self.pdisk.getVolumeUsers(self.persistentDiskUUID)
            if self.instanceNumber > available:
                printError(
                    'disk cannot be attached; it is already mounted (%s/%s)' % (available, self.instanceNumber))
        except AttributeError:
            printError('Persistent disk service unavailable', exit=False)
            raise
        except Exception as e:
            printError(e, exit=False)
            raise

    def _setReadonlyDiskOptional(self):
        if hasattr(self, 'readonlyDiskId') and self.readonlyDiskId:
            self._checkImageExists(self.readonlyDiskId)
            self.readonlyDiskId = self._prependMarketplaceUrlIfImageId(self.readonlyDiskId)
            self.readonly_disk = (self.readonlyDiskId and Runner.READONLY_DISK % self.__dict__) or ''

    def _setInboundPorts(self):
        if Image.isImageId(self.vm_image):
            image = Image(self.configHolder)
            try:
                self.inboundPorts = image.getInboundPortsByImageId(self.vm_image)
            except Exceptions.ExecutionException:
                pass

    @staticmethod
    def getDefaultInstanceTypes():
        types = {
            # name      :   (cpu, ram, swap)
            't1.micro': (1, 512, 512),
            'm1.small': (1, 1536, 1536),
            'm1.medium': (1, 3072, 3072),
            'm1.large': (2, 6144, 6144),
            'm1.xlarge': (4, 8192, 8192),
            'c1.medium': (2, 1536, 1536),
            'c1.xlarge': (4, 6144, 6144),
        }
        return types

    @staticmethod
    def getVmTemplatesParameters(instance=None):
        vmTemplate = Runner.getTemplatePath(instance)

        fd = open(vmTemplate, 'rb')
        template = fd.read()
        fd.close()

        return [Runner._extractTokenName(token) for token in Runner._findTokensInTemplate(template)]

    @staticmethod
    def getTemplatePath(instance=None):
        # TODO: Does this work on Windows?
        vmTemplate = ''
        if instance and hasattr(instance, 'vmTemplateFile'):
            vmTemplate = instance.vmTemplateFile
        if not os.path.exists(vmTemplate):
            vmTemplate = os.path.join(Defaults.SHARE_DIR, 'vm/schema.one')
        if not os.path.exists(vmTemplate):
            vmTemplate = '%s/../../../share/vm/schema.one' % Util.modulePath
        if not os.path.exists(vmTemplate):
            vmTemplate = '%s/../../../src/main/resources/share/vm/schema.one' % Util.modulePath
        return vmTemplate

    @staticmethod
    def _findTokensInTemplate(template):
        return re.findall('%\(\w+\)s', template)

    @staticmethod
    def _extractTokenName(token):
        return re.sub(r'%\((\w+)\)s', r'\1', token)

    @staticmethod
    def defaultRunOptions():

        _sshPublicKey = os.getenv('STRATUSLAB_KEY', Defaults.sshPublicKeyLocation)
        _sshPrivateKey = _sshPublicKey.strip('.pub')
        defaultOp = {'userPublicKeyFile': _sshPublicKey,
                     'userPrivateKeyFile': _sshPrivateKey,
                     'instanceNumber': 1,
                     'instanceType': Runner.DEFAULT_INSTANCE_TYPE,
                     'vmTemplateFile': Runner.getTemplatePath(),
                     'rawData': '',
                     'vmKernel': '',
                     'vmRamdisk': '',
                     'vmName': '',
                     'vmCpuAmount': None,
                     'vmCpu': None,
                     'vmRam': None,
                     'vmSwap': None,
                     'vmDisksBus': Runner.vmDisksBus,
                     'vmRequirements': '',
                     'isLocalIp': False,
                     'isPrivateIp': False,
                     'extraContextFile': '',
                     'extraContextData': '',
                     'cloudInit': '',
                     # FIXME: hack to fix a weird problem with network in CentOS on Fedora 14 + KVM.
                     #        Network is not starting unless VNC is defined. Weird yeh...? 8-/
                     'vncPort': '-1',
                     #'vncPort': None,

                     'vncListen': '',
                     'specificAddressRequest': None,
                     'diskFormat': 'raw',
                     'saveDisk': False,
                     'inVmIdsFile': None,
                     'outVmIdsFile': None,
                     'noCheckImageUrl': False,
                     'msgRecipients': [],
                     'marketplaceEndpoint': Defaults.marketplaceEndpoint,
                     'authorEmail': ''}
        defaultOp.update(CloudEndpoint.options())
        defaultOp.update(PDiskEndpoint.options())
        return defaultOp

    def getInstanceResourceValues(self):

        try:
            cpu, ram, swap = self.availableInstanceTypes.get(self.instanceType)
        except AttributeError:
            cpu, ram, swap = self.getDefaultInstanceTypes().get(self.instanceType)
        if self.vmCpu is not None:
            cpu = self.vmCpu
        if self.vmRam is not None:
            ram = self.vmRam
        if self.vmSwap is not None:
            swap = self.vmSwap

        return cpu, ram, swap

    def _buildVmTemplate(self, template):
        baseVmTemplate = Util.fileGetContent(template)

        self._manageCpuRamSwap()
        self._manageVmName()
        self._manageOsOptions()
        self._manageNetwork()
        self._manageRawData()
        self._manageExtraContext()
        self._manageVnc()
        self._manageNotifications()
        self._manageCreateImage()
        self._manageRequirements()
        self._manageInboundPorts()

        return baseVmTemplate % self._vmParamDict()

    def _vmParamDict(self):
        params = {}
        for param in self.getVmTemplatesParameters(self):
            params[param] = getattr(self, param, '')

        return params

    def _manageInboundPorts(self):
        if self.inboundPorts:
            self.inbound_ports = 'INBOUND_PORTS = "%s"' % ','.join(self.inboundPorts)
        else:
            self.inbound_ports = ''

    def _manageCpuRamSwap(self):
        self.vm_cpu, self.vm_ram, self.vm_swap = self.getInstanceResourceValues()
        self.vm_vcpu = self.vm_cpu
        if self.vmCpuAmount and self.vmCpuAmount <= self.vm_cpu:
            self.vm_cpu = self.vmCpuAmount

    def _manageVmName(self):
        if self.vmName:
            self.vm_name = 'NAME = "%s"' % self.vmName
        else:
            self.vm_name = ''

    def _manageRequirements(self):
        if self.vmRequirements:
            self.vm_requirements = 'REQUIREMENTS = "%s"' % self.vmRequirements
        else:
            self.vm_requirements = ''

    def _manageOsOptions(self):
        if not self.vmKernel and not self.vmRamdisk:
            return

        if self.vmKernel:
            self.os_options += 'kernel = "%s"' % self.vmKernel

        if self.vmRamdisk:
            if self.vmKernel:
                self.os_options += ', '
            self.os_options += 'initrd = "%s"' % self.vmRamdisk

        self.os_options = 'OS = [ %s ]' % self.os_options

    def _manageNetwork(self):
        networkName = self._getNetworkName()
        networkPrefix = 'NIC = [ network_uname=oneadmin,network = "%s" ' % networkName
        networkPostfix = ']\n'
        if self.specificAddressRequest:
            self.vm_nic = networkPrefix + ',\nIP = "%s"' % self.specificAddressRequest + networkPostfix
        else:
            self.vm_nic = networkPrefix + networkPostfix

    def _getNetworkName(self):
        networkName = 'public'
        if self.isLocalIp:
            networkName = 'local'
        elif self.isPrivateIp:
            networkName = 'private'
        return networkName

    def _manageRawData(self):
        if self.rawData:
            if os.path.isfile(self.rawData):
                dataFile = open(self.rawData, 'rb')
                self.rawData = dataFile.read()
                dataFile.close()
            self.rawData = re.escape(self.rawData)
            hypervisor = 'kvm'
            self.raw_data = 'RAW = [ type="%s", data="%s" ]' % (hypervisor,
                                                                self.rawData)

    def _manageExtraContext(self):
        extraContext = {}
        contextElems = []

        if self.extraContextFile:
            contextFile = open(self.extraContextFile, 'rb')
            contextFileData = contextFile.read()
            contextFile.close()
            contextElems.extend(contextFileData.split('\n'))

        if self.extraContextData:
            contextElems.extend(self.extraContextData.split(Util.cliLineSplitChar))

        if self.cloudInit:
            cloudInitArgs = self.cloudInit.split(Util.cliLineSplitChar)
            cloudInitData = CloudInitUtil.contextFile(cloudInitArgs)
            contextElems.extend(cloudInitData.split('\n'))

        for line in contextElems:
            if len(line) == 0:
                continue

            contextLine = line.split('=')

            if len(contextLine) < 2:
                printError('Error while parsing contextualization file.\n'
                           'Syntax error in line `%s`' % line)

            extraContext[contextLine[0]] = '='.join(contextLine[1:])

        contextData = ['%s = "%s",' % (key, value) for key, value in extraContext.items()]

        self._appendContextData(contextData)

    def _appendContextData(self, context):
        self.extra_context += '\n'.join(context)

    def _manageVnc(self):
        vncInfo = []

        if self.vncPort:
            vncInfo.append('port = "%s"' % self.vncPort)

        if self.vncListen:
            vncInfo.append('listen = "%s"' % self.vncListen)

        if len(vncInfo) > 0:
            vncInfo.append('type = "vnc"')

            self.graphics = 'GRAPHICS = [\n%s\n]' % (',\n'.join(vncInfo))

    def _formatRecipient(self, recipient):
        # fields are: 'host', 'vhost', 'user', 'pass', 'queue'
        # badly formatted inputs are silently ignored
        values = recipient.split(',')
        return Runner.NOTIFICATION.format(*values) if (len(values) == 5) else ''

    def _manageNotifications(self):
        if self.msgRecipients:
            notificationInfo = map(self._formatRecipient, self.msgRecipients)
            self.notifications = ('\n'.join(notificationInfo))
        else:
            self.notifications = ''

    def _manageCreateImage(self):
        if not self.saveDisk:
            return

        data = ['%s = "%s"' % (k, v) for k, v in self.createImageData.items()]
        self.create_image = Runner.CREATE_IMAGE % ',\n'.join(data)

    def updateCreateImageTemplateData(self, updateDict):
        self.createImageData.update(updateDict)

    def runInstance(self, details=False):
        self._printContacting()

        if Image.isImageId(self.vm_image):
            self._checkImageExists(self.vm_image)
            self.vm_image = self._prependMarketplaceUrlIfImageId(self.vm_image)
        elif Image.isDiskId(self.vm_image):
            self.vm_image = self._createDiskUrlIfDiskId(self.vm_image)
        elif self._isAliasUrl(self.vm_image):
            self.vm_image = self._resolveUrl(self.vm_image)
        else:
            raise Exceptions.ValidationException('Image reference must be an '
                                                 'Alias URL, Marketplace Image ID or Disk ID:  %s' %
                                                 self.vm_image)

        printAction('Starting machine(s)')

        self.printDetail('Using VM template file: %s' % self.vmTemplateFile)

        vmTpl = self._buildVmTemplate(self.vmTemplateFile)

        label = (self.instanceNumber > 1) and 'machines' or 'machine'

        printStep('Starting %s %s' % (self.instanceNumber, label))

        self.printDetail('on endpoint: %s' % self.endpoint, Util.VERBOSE_LEVEL_DETAILED)
        self.printDetail('with template:\n%s' % vmTpl, Util.VERBOSE_LEVEL_DETAILED)

        for vmNb in range(self.instanceNumber):
            vmId = self.cloud.vmStart(vmTpl)
            self.vmIds.append(vmId)
            networkName, ip = self.getNetworkDetail(vmId)
            self.vmIdsAndNetwork.append((vmId, networkName, ip))
            vmIpPretty = '\t%s ip: %s' % (networkName.title(), ip)
            printStep('Machine %s (vm ID: %s)\n%s' % (vmNb + 1, vmId, vmIpPretty))
            self.instancesDetail.append({'id': vmId, 'ip': ip, 'networkName': networkName})
        self._saveVmIds()

        printStep('Done!')

        if not details:
            return self.vmIds
        else:
            return self.vmIdsAndNetwork

    def save_instance_as_new_image(self, vm_id):
        self._printContacting()
        self._checkInstanceExists(vm_id)

        printStep('Instructing cloud to save instance as new image on shutdown')

    def getNetworkDetail(self, vmId):
        networkName, ip = self.cloud.getVmIp(vmId)
        return networkName, ip

    def _saveVmIds(self):
        if self.outVmIdsFile:
            open(self.outVmIdsFile, 'w').write('\n'.join(map(str, self.vmIds)))

    def _loadVmIdsFromFile(self):
        vmIds = []

        if self.inVmIdsFile:
            vmIds = open(self.inVmIdsFile).read().split('\n')

        return vmIds

    def killInstances(self, ids=[]):
        self._operateOnInstance('Kill', ids=ids)

    def shutdownInstances(self, ids=[]):
        self._operateOnInstance('Shutdown', ids=ids)

    def _isAliasUrl(self, url):
        # TODO: Put in real implementation.
        return True

    def _operateOnInstance(self, operation, ids=[]):
        operations = ('Shutdown', 'Kill')
        if operation not in operations:
            raise Exceptions.ExecutionException('Unsupported operation on instance: %s' %
                                                operation)
        vmIds = ids or self.vmIds
        if self.inVmIdsFile:
            vmIds = self._loadVmIdsFromFile()
        if operation == 'Shutdown':
            instance_operation = self.cloud.vmStop
        if operation == 'Kill':
            instance_operation = self.cloud.vmKill
        for vmId in vmIds:
            self.printDetail('Sending "%s" request for instance %s.' % (operation, vmId))
            instance_operation(int(vmId))
        plural = (len(vmIds) > 1 and 's') or ''
        self.printDetail('"%s" %s VM%s: %s' % (operation, len(vmIds), plural,
                                               ', '.join(map(str, vmIds))))

    def printDetail(self, msg, verboseLevel=Util.VERBOSE_LEVEL_NORMAL):
        return Util.printDetail(msg, self.verboseLevel, verboseLevel)

    def waitUntilVmRunningOrTimeout(self, vmId, vmStartTimeout=120, failOn=()):
        return self.cloud.waitUntilVmRunningOrTimeout(vmId, vmStartTimeout, failOn=failOn)

    def getVmState(self, vmId):
        return self.cloud.getVmState(vmId)

    def _checkImageExists(self, imageId):
        self.printDetail('Checking image availability.')
        if self.noCheckImageUrl:
            printWarning('Image availability check is disabled.')
            return
        imageObject = Image(self.configHolder)
        imageObject.checkImageExists(imageId)

    def _prependMarketplaceUrlIfImageId(self, image):
        if Image.isImageId(image):
            return MarketplaceUtil.metadataUrl(self.marketplaceEndpoint, image)
        else:
            return image

    def _createDiskUrlIfDiskId(self, image):
        if Image.isDiskId(image):
            self.pdiskEndpointHostname = PersistentDisk.getFQNHostname(self.pdiskEndpoint)
            return "pdisk:%s:%s:%s" % (self.pdiskEndpointHostname, self.pdiskPort, image)
        else:
            return image

    def _printContacting(self):
        self.printDetail('Accessing compute service at: %s' % self.endpoint)

    def _resolveUrl(self, url):
        """Uses a HEAD request to resolve an http(s) URL with a possible redirect."""
        if url.startswith("http"):
            response = urllib2.urlopen(Runner.HeadRequest(url))
            return response.geturl()
        else:
            return url

    def listInstanceTypes(self):
        types = self.availableInstanceTypes

        columnSize = 10

        output = ''
        output += ' '.ljust(1)
        output += 'Type'.ljust(columnSize)
        output += 'CPU'.rjust(columnSize)
        output += 'RAM'.rjust(columnSize)
        output += 'SWAP'.rjust(columnSize)
        output += "\n"
        for name in sorted(types.iterkeys()):
            flag = (name == self.instanceType and '*') or ' '
            cpu, ram, swap = types[name]
            output += "%s %s %s %s %s\n" % (flag.ljust(1),
                                            name.ljust(columnSize),
                                            ('%s CPU' % cpu).rjust(columnSize),
                                            ('%s MB' % ram).rjust(columnSize),
                                            ('%s MB' % swap).rjust(columnSize))

        return output



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import tempfile

from stratuslab import Util
from stratuslab.ManifestInfo import ManifestInfo
from stratuslab.Signator import Signator
from stratuslab.ConfigHolder import ConfigHolder
from stratuslab.AuthnCommand import P12Certificate

from stratuslab.marketplace.Uploader import Uploader

from Util import Util as MarketplaceUtil
from stratuslab.Exceptions import ExecutionException
from stratuslab.marketplace.ManifestDownloader import ManifestDownloader

etree = Util.importETree()

class Deprecator(object):

    @staticmethod
    def buildDeprecatorParser(parser):
        parser.usage = '''usage: %prog [options] image-id'''

        parser.description = '''
Create a new metadata entry for the given image that deprecates the
image.  The reason for the deprecation must be given. The image-id
argument is the identifier of the image to deprecate.
'''

        MarketplaceUtil.addEndpointOption(parser)
        P12Certificate.addOptions(parser)

        parser.add_option('--email', dest='email',
                help='email address of endorser of metadata entry',
                default='')

        parser.add_option('--created', dest='created',
                help='date of metadata entry to be deprecated, latest entry will be deprecated if not provided',
                default='')

        parser.add_option('--reason', dest='reason',
                help='Comment to append to deprecation',
                default='This image has been deprecated', metavar='TEXT')

    @staticmethod
    def checkDeprecatorOptions(options, parser):
        MarketplaceUtil.checkEndpointOption(options)

        if not P12Certificate.checkOptions(options):
            parser.error('Missing credentials. Please provide %s' % P12Certificate.optionString)

        if not options.email:
            parser.error('Missing email address. Please provide email of endorser')

    def __init__(self, configHolder=ConfigHolder()):
        self.configHolder = configHolder

        configHolder.assign(self)

        self.uploader = Uploader(configHolder)
        self.manifestDownloader = ManifestDownloader(configHolder)

    def deprecate(self, imageId):
        tempDeprecatedMetadataFilename = tempfile.mktemp()
        tempMetadataFilename = tempfile.mktemp()
        try:
            imageURI = imageId + '/' + self.email
            if len(self.created) != 0:
                imageURI = imageURI + '/' + self.created

            # Get metadata file
            self.manifestDownloader.getManifestAsFile(imageURI, tempMetadataFilename)

            # Strip signature
            xml = etree.ElementTree(file=tempMetadataFilename)
            root = xml.getroot()

            if (root.tag == "metadata"):
                rootElement = root.find('.//{%s}RDF' % ManifestInfo.NS_RDF)
            else:
                rootElement = root

            descriptionElement = rootElement.find('.//{%s}Description' % ManifestInfo.NS_RDF)
            descriptionElement.remove(descriptionElement.find('.//{%s}endorsement' % ManifestInfo.NS_SLREQ))
            endorsement = etree.Element('{%s}%s' % (ManifestInfo.NS_SLREQ, 'endorsement'), parseType="Resource")
            descriptionElement.append(endorsement)

            signatureElement = rootElement.find('.//{%s}Signature' % 'http://www.w3.org/2000/09/xmldsig#')
            rootElement.remove(signatureElement)

            xml._setroot(rootElement)

            # Add deprecated entry
            elem = etree.Element('{%s}%s' % (ManifestInfo.NS_SLTERMS, 'deprecated'))
            elem.text = self.reason
            descriptionElement.append(elem)

            xml.write(tempDeprecatedMetadataFilename)

            # Sign and upload
            signator = Signator(tempDeprecatedMetadataFilename, self.configHolder)

            isError = signator.sign()
            if isError:
                raise ExecutionException('Error signing new manifest')
            
            return self.uploader.upload(tempDeprecatedMetadataFilename)
        finally:
            try:
                os.unlink(tempMetadataFilename)
                os.unlink(tempDeprecatedMetadataFilename)
                os.unlink(tempDeprecatedMetadataFilename+'.orig')
            except:
                pass



#
# Copyright (c) 2013, Centre National de la Recherche Scientifique (CNRS)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
This module provides standard logging functions for the StratusLab
API.  It provides convenience functions that allow a console or
syslog logger to be obtained easily.  It also allows the special
'stratuslab.api' logger to be obtained and modified.

Note that the 'stratuslab.api' logger is configured by default to
do nothing.  If you want to enable the logging, then this module
should be imported and the appropriate handlers added to the logger.
This can be done easily with the following:

>>> import stratuslab.api.LogUtil as LogUtil
>>> LogUtil.get_console_logger()
>>> ...
>>> LogUtil.set_logger_level(logging.DEBUG)

Name of the logger instance used throughout the StratusLab API.  The
logger associated with this name should only be manipulated through
the utility functions defined here.
"""

import sys
import logging
from logging import StreamHandler
from logging.handlers import SysLogHandler


STRATUSLAB_API_LOGGER_NAME = 'stratuslab.api'


class NullHandler(logging.Handler):
    """
    This handler acts as a 'no-op'.  It is a placeholder for the API
    to avoid having logging complain that there is no handler defined.
    This class is used only for Python 2.6 because in Python 2.7 such
    a class was defined in the standard distribution.
    """

    lock = None

    def __init__(self, level=logging.INFO):
        # NOTE: super cannot be used here because it will fail on
        # python 2.6, which uses old style classes for logging.
        logging.Handler.__init__(self, level=level)

    def emit(self, record):
        pass

    def handle(self, record):
        pass

    def createLock(self):
        return self.lock


def get_console_logger(name=STRATUSLAB_API_LOGGER_NAME, level=logging.INFO, fmt=None):
    """
    Provides a logger that sends messages to the standard output (i.e.
    console) with the given name.  If the name is not supplied, the
    STRATUSLAB_API_LOGGER_NAME is used.  If the given logger already
    exists, then the configured console logger will be added to it.
    The logging level is set to INFO if not supplied.  The default
    logging format will just emit the message.
    """

    logger = logging.getLogger(name)

    handler = StreamHandler(sys.stdout)
    if not fmt:
        fmt = logging.Formatter(fmt=None, datefmt=None)
    handler.setFormatter(fmt)

    logger.setLevel(level)
    logger.addHandler(handler)
    return logger


def get_syslog_logger(name=STRATUSLAB_API_LOGGER_NAME, level=logging.INFO):
    """
    Provides a syslog-configured logger with the given name.  If the
    name is not supplied then STRATUSLAB_API_LOGGER_NAME is used.  If
    the given logger already exists, then the syslog handler will be
    added to it.  The logging level will be set to INFO if not
    supplied.
    """

    logger = logging.getLogger(name)

    handler = SysLogHandler(address='/dev/log')
    logger.setLevel(level)
    logger.addHandler(handler)
    return logger


def initialize_logger(logger_name=STRATUSLAB_API_LOGGER_NAME, level=logging.INFO):
    """
    Initializes the logger associated with the given name with a
    handler that will print nothing (NullHandler).  It removes any
    existing handlers and resets the logging level to INFO or to the
    level given.
    """
    logger = logging.getLogger(logger_name)
    for h in list(logger.handlers):
        logger.removeHandler(h)

    try:
        h = logging.NullHandler(level=level)  # for python 2.7.x
    except AttributeError:
        h = NullHandler(level=level)  # for python 2.6.x

    logger.addHandler(h)


def set_logger_level(logger_name=STRATUSLAB_API_LOGGER_NAME, level=logging.INFO):
    """
    Sets or resets the logging level for the named logger.  If the
    name is not supplied, then STRATUSLAB_API_LOGGER_NAME will be
    used.
    """
    logger = logging.getLogger(logger_name)
    logger.setLevel(level)


def critical(msg):
    """
    Convenience method for logging critical messages to the StratusLab
    API logger.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.critical(msg)


def error(msg):
    """
    Convenience method for logging error messages to the StratusLab
    API logger.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.error(msg)


def warning(msg):
    """
    Convenience method for logging warning messages to the StratusLab
    API logger.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.warning(msg)


def info(msg):
    """
    Convenience method for logging info messages to the StratusLab API
    logger.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.info(msg)


def debug(msg):
    """
    Convenience method for logging debug messages to the StratusLab
    API logger.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.debug(msg)


def exception(msg):
    """
    Convenience method for logging exception messages to the
    StratusLab API logger.  This method should only be called from
    exception handlers.  The exception information is included in the
    message.
    """
    logger = logging.getLogger(STRATUSLAB_API_LOGGER_NAME)
    logger.exception(msg)


#
# Initialize the logger with a 'no-op' handler.  Add another handler
# to actually push the logging messages somewhere.
#
initialize_logger()



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from BaseSystem import BaseSystem
from stratuslab.system.PackageInfo import PackageInfo
import stratuslab.Util as Util

installCmd = 'apt-get update && apt-get -y --force-yes install'
updateCmd = 'apt-get update'
cleanPackageCacheCmd = 'apt-get clean'
queryPackageCmd = 'dpkg -s'

class Ubuntu(BaseSystem):

    os = 'ubuntu'

    def __init__(self):
        self.systemName = 'Ubuntu 10.04'
        self.installCmd = installCmd
        self.frontendDeps = [
            'ruby', 'libsqlite3-dev', 'libxmlrpc-c3-dev', 'libssl-dev',
            'scons', 'g++', 'git-core', 'ssh', 'genisoimage', 'curl', 'libxml2-dev'
        ]
        self.nodeDeps = ['ssh', 'ruby', 'curl', 'libvirt-bin', 'genisoimage' ]
        self.hypervisorDeps = {
            'xen': ['xen-hypervisor-3.3'],
            'kvm': ['qemu-kvm'],
        }
        self.fileSharingFrontendDeps = {
            'nfs': ['nfs-kernel-server'],
            'ssh': [],
        }
        self.fileSharingNodeDeps = {
            'nfs': ['nfs-common'],
            'ssh': [],
        }

        self.packages = {'apache2': PackageInfo('apache2','/etc/apache2'),
                         'dhcp': PackageInfo('dhcp3-server',
                                             configFile='/etc/dhcp3/dhcpd.conf',
                                             initdScriptName='dhcp3-server'),
                        'MySQLServer': PackageInfo('mysql-server',
                                                   initdScriptName='mysql')}

        super(Ubuntu, self).__init__()

    # -------------------------------------------
    #     Package manager and related
    # -------------------------------------------

    def updatePackageManager(self):
        self._execute(['apt-get', 'update'])

    def getIsPackageInstalledCommand(self, package):
        cmd = "%s %s | grep Status | grep -q 'ok installed'" % (queryPackageCmd,
                                                                package)
        return cmd

    # -------------------------------------------
    #     Hypervisor related methods
    # -------------------------------------------

    def _configureKvm(self):
        super(Ubuntu, self)._configureKvm()
        self.executeCmd(['/etc/init.d/libvirt-bin start'])
        self.executeCmd(['usermod', '-G', 'libvirtd', '-a', self.oneUsername])

    # -------------------------------------------
    # Network related methods
    # -------------------------------------------

    FILE_INTERFACES = '/etc/network/interfaces'
    # re-defining for ubuntu
    FILE_FIREWALL_RULES = '/etc/iptables.rules'

    def _configureNetworkInterface(self, device, ip, netmask):
        data = """auto %s
iface %s inet static
  address %s
  netmask %s
  pre-up iptables-restore < %s""" % (device, device, ip, netmask,
                                     self.FILE_FIREWALL_RULES)

        Util.appendOrReplaceMultilineBlockInFile(self.FILE_INTERFACES, data)

    # -------------------------------------------
    # CA
    # -------------------------------------------

    def _installCAs(self):
        """CA:
        https://wiki.egi.eu/wiki/EGI_IGTF_Release
        http://repository.egi.eu/sw/production/cas/1/current/tgz/
        """
        Util.printDetail('NB! Installation of CA is not implemented for Ubuntu.')
        Util.printDetail("""For manual installation see:
        https://wiki.egi.eu/wiki/EGI_IGTF_Release
        http://repository.egi.eu/sw/production/cas/1/current/tgz/""")
        self._installFetchCrl()
        
    def _installFetchCrl(self):
        """fetch-crl 3:
        http://www.nikhef.nl/grid/fetchcrl3
        http://dist.eugridpma.info/distribution/util/fetch-crl3/
        """
        Util.printDetail('NB! Installation of fetch-crl is not implemented for Ubuntu.')
        Util.printDetail("""For manual installation see:
        http://www.nikhef.nl/grid/fetchcrl3
        http://dist.eugridpma.info/distribution/util/fetch-crl3/""")

system = Ubuntu()



from .Defaults import MSG_CLIENTS

def getMsgClient(configHolder):
    return MsgClientFactory(configHolder).getClient()

class MsgClientFactory(object):

    def __init__(self, configHolder):
        configHolder.assign(self)
        self.configHolder = configHolder

    def getClient(self):
        msg_type = self.configHolder.msg_type.lower()

        if msg_type not in MSG_CLIENTS.keys():
            raise Exception('Unknown messaging client type: %s' % msg_type)

        MsgClient = None
        exec "from stratuslab.messaging.%(msg_client)s import %(msg_client)s as MsgClient" % \
                                            {'msg_client': MSG_CLIENTS[msg_type]}
        return MsgClient(self.configHolder)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2012, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from dirq.QueueSimple import QueueSimple
from stratuslab.messaging.MsgBase import MsgBase

class DirectoryQueue(MsgBase):
    def __init__(self, configHolder):
        super(DirectoryQueue, self).__init__(configHolder)

        self.queue = QueueSimple(self.msg_queue)

    def send(self, message):
        self.queue.add(message)



#!/usr/bin/env python
#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, Centre National de la Recherche Scientifique (CNRS)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from datetime import datetime
from datetime import timedelta
import json
import re
from urllib import urlencode
from socket import getfqdn

from stratuslab.HttpClient import HttpClient
from uuid import UUID
from stratuslab.Util import printError
import Util
from stratuslab import Defaults
from stratuslab.Authn import UsernamePasswordCredentialsLoader
from stratuslab.Exceptions import ValidationException, ClientException, \
    ServerException, ConfigurationException
from stratuslab.ConfigHolder import ConfigHolder


class PersistentDisk(object):
    def __init__(self, configHolder=ConfigHolder()):
        self.pdiskUsername = None
        self.pdiskPassword = None
        self.username = None
        self.password = None
        self.pemCertificate = None
        self.pemKey = None
        self.verboseLevel = Util.VERBOSE_LEVEL_NORMAL
        self.pdiskEndpoint = None
        self.endpointSuffix = ''
        self.maxMounts = 10

        self.configHolder = configHolder
        self.configHolder.assign(self)

        # This will eventually contain the sanitized endpoint
        # with the proper suffix attached for the authentication
        # method being used.
        self.endpoint = None

        if not self.pdiskEndpoint:
            try:
                self.pdiskEndpoint = configHolder.endpoint
            except AttributeError:
                raise ConfigurationException('Missing persistent disk endpoint.')

    def _initPDiskConnection(self, configHolder=None):
        self.client = HttpClient(configHolder or self.configHolder)
        self._addCredentials()
        self._buildFQNEndpoint()

    def _getJson(self, url):
        return self.client.get(url, accept='application/json')

    def _getGzip(self, url):
        return self.client.get(url, accept='application/x-gzip')

    def _postJson(self, url, body=None, contentType='application/x-www-form-urlencoded'):
        headers, content = self.client.post(url,
                                            body,
                                            contentType,
                                            accept='application/json')
        return headers, content.replace('\\', '')

    def _putJson(self, url, body, contentType='application/x-www-form-urlencoded'):
        self.client.put(url,
                        body,
                        contentType,
                        accept='application/json')

    def _postMultipart(self, url, files, params=[]):
        headers, content = self.client.post_multipart(url,
                                                      files,
                                                      params,
                                                      accept='application/json')
        return headers, content.replace('\\', '')

    def describeVolumes(self, filters={}):
        self._initPDiskConnection()
        self._printContacting()
        listVolUrl = '%s/disks/' % self.endpoint
        headers, jsonDiskList = self._getJson(listVolUrl)
        self._raiseOnErrors(headers, jsonDiskList)
        disks = json.loads(jsonDiskList)
        return self._filterDisks(disks, filters)

    def search(self, key, value):
        self._setPDiskUserCredentials()
        filtered = self.describeVolumes({key: value})
        return [disk['uuid'] for disk in filtered]

    def quarantineVolume(self, uuid):
        self._setPDiskUserCredentials()
        keyvalues = {'owner': self.pdiskUsername,
                     'quarantine': datetime.now()}
        self.updateVolume(keyvalues, uuid)

    def updateVolume(self, keyvalues, uuid):
        # Need to set the user as pdisk since we need to be super
        # to update pdisk metadata
        self._setPDiskUserCredentials()
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s' % (self.endpoint, uuid)
        body = urlencode(keyvalues)
        self._putJson(url, body)

    def updateVolumeAsUser(self, keyvalues, uuid):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s' % (self.endpoint, uuid)
        body = urlencode(keyvalues)
        self._putJson(url, body)

    def getValue(self, key, uuid):
        self._setPDiskUserCredentials()
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s' % (self.endpoint, uuid)
        headers, jsonDisk = self._getJson(url)
        self._raiseOnErrors(headers, jsonDisk)
        disk = json.loads(jsonDisk)
        return disk[key]

    def _setPDiskUserCredentials(self):
        """Assign the super pdisk username/password"""
        if self.pdiskUsername and self.pdiskPassword:
            return
        loader = UsernamePasswordCredentialsLoader()
        loader.load()
        self.pdiskUsername = self.persistentDiskCloudServiceUser
        self.pdiskPassword = loader.get_password(self.pdiskUsername)

    def createVolume(self, size, tag, visibility):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/' % self.endpoint
        body = {'size': size,
                'tag': tag,
                'visibility': self._getVisibilityFromBool(visibility)}
        headers, uuid = self._postJson(url, urlencode(body))
        if headers.status == 201:
            return self._getUuidFromJson(uuid)
        self._raiseOnErrors(headers, uuid)

    def createVolumeFromUrl(self, size, tag, visibility, imageUrl, bytes, sha1):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/' % self.endpoint
        body = {'size': size,
                'tag': tag,
                'visibility': self._getVisibilityFromBool(visibility),
                'url': imageUrl,
                'bytes': bytes,
                'sha1': sha1}
        headers, uuid = self._postJson(url, urlencode(body))
        if headers.status == 201:
            return self._getUuidFromJson(uuid)
        self._raiseOnErrors(headers, uuid)

    def createCowVolume(self, uuid, tag):
        # TODO: add iscow check
        self._setPDiskUserCredentials()
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s' % (self.endpoint, uuid)
        body = None
        # FIXME: We can't set body and use redirect "See Other" RPC pattern, as
        #        there seems to be a bug in httplib2, which incorrectly approximates
        #        the accepted MIME type when doing GET on redirected resource.
        #        See: http://jira.stratuslab.eu:8080/browse/STRATUSLAB-941
        #        if tag:
        #            body = urlencode({'tag': tag})
        try:
            _, content = self._postJson(url, body)
        except Exception, ex:
            ex.mediaType = 'json'
            raise
        return self._getUuidFromJson(content)

    def rebaseVolume(self, uuid):
        self._setPDiskUserCredentials()
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s' % (self.endpoint, uuid)
        _, content = self._postJson(url)
        return self._getUuidFromJson(content)

    def deleteVolume(self, uuid):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s/' % (self.endpoint, uuid)
        headers, uuid = self.client.delete(url, accept="application/json")
        self._raiseOnErrors(headers, uuid)
        return self._getUuidFromJson(uuid)

    def volumeExists(self, uuid):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s/' % (self.endpoint, uuid)
        headers, _ = self.client.head(url)
        return headers.status == 200

    def getVolumeUsers(self, uuid):
        count = self.getVolumeUserCount(uuid)
        return self.maxMounts - count, self.maxMounts

    def getVolumeUserCount(self, uuid):
        self._initPDiskConnection()
        self._printContacting()
        volumeUrl = '%s/disks/%s/' % (self.endpoint, uuid)
        headers, content = self._getJson(volumeUrl)
        self._raiseOnErrors(headers, content)
        return int(json.loads(content)['count'])

    def getTurl(self, uuid):
        self._setPDiskUserCredentials()
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s/turl/' % (self.endpoint, uuid)
        headers, content = self._getJson(url)
        self._raiseOnErrors(headers, content)
        return json.loads(content)['turl']

    def hotAttach(self, node, vmId, uuid):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s/mounts/' % (self.endpoint, uuid)
        body = {'node': node,
                'vm_id': vmId}
        headers, content = self._postJson(url, urlencode(body))
        self._raiseOnErrors(headers, content)
        return json.loads(content)['target']

    def hotDetach(self, vmId, uuid):
        self._initPDiskConnection()
        self._printContacting()
        url = '%s/disks/%s/mounts/%s_%s' % (self.endpoint, uuid, uuid, vmId)
        headers, content = self.client.delete(url, accept="application/json")
        self._raiseOnErrors(headers, content)
        return json.loads(content)['target']

    def downloadVolume(self, uuid, filename):
        # Don't cache http response as it will contain the image in the body.
        configHolder = self.configHolder.copy()
        configHolder.set('useHttpCache', False)
        self._initPDiskConnection(configHolder=configHolder)
        self._printContacting()
        volumeUrl = '%s/disks/%s/' % (self.endpoint, uuid)
        headers, content = self._getGzip(volumeUrl)
        self._raiseOnErrors(headers, content)
        Util.filePutContent(filename, content, True)

    def uploadVolume(self, filename):
        """Upload compressed image and return resource URL."""
        self._initPDiskConnection()
        self._printContacting()
        base_url = '%s/disks/' % self.endpoint
        headers, content = self._uploadImageFile(base_url, filename)
        self._raiseOnErrors(headers, content)
        uuid = self._getUuidFromJson(content)
        return base_url.strip('/') + '/' + uuid

    def _uploadImageFile(self, url, filename):
        return self._postMultipart(url, [("Image File", open(filename))])

    def serviceAvailable(self):
        try:
            self._initPDiskConnection()
            self._printContacting()
            header, _ = self.client.head('%s' % self.endpoint)
            if header.status == 200:
                return True
            return False
        except Exception:
            return False

    def _raiseOnErrors(self, headers, content):
        if headers.status in (400, 411):
            raise Exception(json.loads(content)['message'])
        if headers.status != 200:
            raise Exception(headers.reason)

    def _getUuidFromJson(self, jsonUuid):
        return json.loads(jsonUuid)['uuid']

    def _getVisibilityFromBool(self, visibility):
        return visibility and 'public' or 'private'

    def _filterDisks(self, disks, filters):
        availableDisk = []
        for disk in disks:
            addDisk = True
            for propertyName, searchedValue in filters.items():
                diskProperty = disk.get(propertyName, None)
                if not diskProperty:
                    addDisk = False
                    break
                for value in searchedValue:
                    if not re.search(value, diskProperty):
                        addDisk = False
                        break
            if addDisk:
                availableDisk.append(disk)
        return availableDisk

    def _addCredentials(self):
        cert = self.pemCertificate
        key = self.pemKey
        user = self.pdiskUsername or self.username
        password = self.pdiskPassword or self.password

        # Must test username/password first because there will
        # always be default values set for the certificate.
        if user and password:
            self.endpointSuffix = '/pswd'
            self.client.addCredentials(user, password)
        elif cert and key:
            self.endpointSuffix = '/cert'
            self.client.addCertificate(key, cert)
        else:
            raise ValueError('Missing credentials')

    def _buildFQNEndpoint(self):
        self.endpoint = Util.sanitizeEndpoint(self.pdiskEndpoint,
                                              Defaults.pdiskProtocol,
                                              Defaults.pdiskPort)
        self.endpoint += self.endpointSuffix

    @staticmethod
    def getFQNHostname(hostname):
        try:
            return getfqdn(hostname)
        except Exception:
            printError('Unable to translate endpoint "%s" to an IP address' % hostname,
                       exit=False)

    @staticmethod
    def isValidUuid(uuid):
        try:
            UUID(uuid)
        except ValueError:
            return False
        return True

    def _printContacting(self):
        self._printDetail('Accessing storage service at: %s' % self.endpoint)

    def _printDetail(self, message):
        Util.printDetail(message, self.verboseLevel, 1)

    def cleanQuarantine(self):
        self._setPDiskUserCredentials()

        disks = self.describeVolumes({'quarantine': ['.*']})
        threshold = self._getQuarantineThresholdDate()

        disksToCleanUp = []

        for disk in disks:
            quarantineDate = self._parseQuarantineDate(disk['quarantine'])
            if quarantineDate < threshold:
                disksToCleanUp.append(disk)
        for disk in disksToCleanUp:
            self._printDetail('Removing disk: %s' % disk['uuid'])
            try:
                self.deleteVolume(disk['uuid'])
            except (ClientException, ServerException), ex:
                Util.printError(str(datetime.now()) + ' ' + str(ex))

    def _getQuarantineThresholdDate(self):
        now = datetime.now()
        quarantineTime = self._getQuarantinePeriod()
        return now - timedelta(minutes=quarantineTime)

    def _getQuarantinePeriod(self):
        period, factor = self._parseQuarantinePeriod()
        return period * factor

    def _parseQuarantinePeriod(self):
        if not self.quarantinePeriod:
            raise ValidationException('Invalid quarantine_period parameter. Cannot be empty.')

        factor = 1

        if self.quarantinePeriod.isdigit():
            return int(self.quarantinePeriod), factor

        factors = {'m': 1, 'h': 60, 'd': 60 * 24}
        unit = self.quarantinePeriod[-1].lower()
        if unit not in factors.keys():
            raise ValidationException(
                'Invalid quarantine_period parameter unit. Should be one of: %s.' % ', '.join(factors.keys()))

        if not self.quarantinePeriod[:-1].isdigit():
            raise ValidationException('Invalid quarantine_period parameter value. Should be integer [+ unit].')

        return int(self.quarantinePeriod[:-1]), factors[unit]

    def _parseQuarantineDate(self, dateString):
        return datetime.strptime(dateString, '%Y-%m-%d %H:%M:%S.%f')



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import getpass

from stratuslab import Defaults

from CommandBase import CommandBaseUser


class AuthnCommand(CommandBaseUser):
    @staticmethod
    def defaultRunOptions():
        options = {}
        options.update(AuthnCommand.userNamePasswordOptions())
        options.update(AuthnCommand.certPemOptions())
        options.update(AuthnCommand.certP12Options())
        # options.update(AuthnCommand.pdiskOptions())
        return options

    @staticmethod
    def userNamePasswordOptions():
        return UsernamePassword.options()

    @staticmethod
    def certPemOptions():
        return PemCertificate.options()

    @staticmethod
    def certP12Options():
        return P12Certificate.options()

    @staticmethod
    def cloudEndpointOptions():
        return CloudEndpoint.options()

    @staticmethod
    def addUsernamePasswordOptions(parser, defaultOptions=None):
        return UsernamePassword.addOptions(parser, defaultOptions)

    @staticmethod
    def addPemCertOptions(parser, defaultOptions=None):
        return PemCertificate.addOptions(parser, defaultOptions)

    @staticmethod
    def addP12CertOptions(parser, defaultOptions=None):
        return P12Certificate.addOptions(parser, defaultOptions)

    @staticmethod
    def addCloudEndpointOptions(parser, defaultOptions=None):
        return CloudEndpoint.addOptions(parser, defaultOptions)

    def parse(self):
        defaultOptions = AuthnCommand.defaultRunOptions()

        AuthnCommand.addUsernamePasswordOptions(self.parser, defaultOptions)
        AuthnCommand.addPemCertOptions(self.parser, defaultOptions)

    def checkPemCertOptions(self):
        return PemCertificate.checkOptions(self.options)

    def checkP12CertOptions(self):
        return P12Certificate.checkOptions(self.options)

    def checkUsernamePasswordOptions(self):
        return UsernamePassword.checkOptions(self.options)

    def checkCloudEndpointOptoins(self):
        return CloudEndpoint.checkOptions(self.options)

    def checkOptions(self):
        if not (self.checkUsernamePasswordOptions() or self.checkPemCertOptions()):
            self.parser.error('Missing credentials. Please provide either %s or %s' %
                              (UsernamePassword.optionString, PemCertificate.optionString))

    def checkPemCertOptionsOnly(self):
        if not self.checkPemCertOptions():
            self.parser.error('Missing credentials. Please provide %s' % PemCertificate.optionString)

    def checkP12CertOptionsOnly(self):
        if not self.checkP12CertOptions():
            self.parser.error('Missing credentials. Please provide %s' % P12Certificate.optionString)

    def checkCloudEndpointOptionsOnly(self):
        if not self.checkCloudEndpointOptoins():
            self.parser.error('Missing cloud endpoint. Please provide %s' % CloudEndpoint.optionString)


class UsernamePassword(object):
    optionString = '--username/--password'

    @staticmethod
    def options():
        return {'username': os.getenv('STRATUSLAB_USERNAME', ''),
                'password': os.getenv('STRATUSLAB_PASSWORD', '')}

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        if not defaultOptions:
            defaultOptions = AuthnCommand.defaultRunOptions()

        parser.add_option('-u', '--username', dest='username',
                          help='cloud username. Default STRATUSLAB_USERNAME',
                          default=defaultOptions['username'])
        parser.add_option('-p', '--password', dest='password',
                          help='cloud password. Default STRATUSLAB_PASSWORD',
                          default=defaultOptions['password'])
        return parser

    @staticmethod
    def checkOptions(options):
        if options.username and options.password:
            return True

        if options.username and not options.password:
            prompt = "'%s' at '%s' password: " % (options.username,
                                                  options.endpoint)
            options.password = getpass.getpass(prompt=prompt)

        return True


class PemCertificate(object):
    optionString = '--pem-cert/--pem-key'

    certDefaultLocation = Defaults.pemCertificateLocation
    keyDefaultLocation = Defaults.pemKeyLocation

    @staticmethod
    def options():
        return {'pemCertificate': os.getenv('STRATUSLAB_PEM_CERTIFICATE',
                                            PemCertificate.certDefaultLocation),
                'pemKey': os.getenv('STRATUSLAB_PEM_KEY',
                                    PemCertificate.keyDefaultLocation)}

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        if not defaultOptions:
            defaultOptions = AuthnCommand.defaultRunOptions()

        parser.add_option('--pem-cert', dest='pemCertificate',
                          help='PEM certificate file. Default %s. STRATUSLAB_PEM_CERTIFICATE' % PemCertificate.certDefaultLocation,
                          default=defaultOptions['pemCertificate'], metavar='FILE')
        parser.add_option('--pem-key', dest='pemKey',
                          help='PEM public key file. Default %s. STRATUSLAB_PEM_KEY' % PemCertificate.keyDefaultLocation,
                          default=defaultOptions['pemKey'], metavar='KEY')
        return parser

    @staticmethod
    def checkOptions(options):
        if options.pemCertificate and options.pemKey:
            return True

        return False


class P12Certificate(object):
    optionString = '--p12-cert/--p12-password'

    certDefaultLocation = Defaults.p12CertificateLocation

    @staticmethod
    def options():
        return {'p12Certificate': os.getenv('STRATUSLAB_P12_CERTIFICATE',
                                            P12Certificate.certDefaultLocation),
                'p12Password': os.getenv('STRATUSLAB_P12_PASSWORD', '')}

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        if not defaultOptions:
            defaultOptions = AuthnCommand.defaultRunOptions()

        parser.add_option('--p12-cert', dest='p12Certificate',
                          help='PKCS12 (P12) certificate file. Default %s. STRATUSLAB_P12_CERTIFICATE' % P12Certificate.certDefaultLocation,
                          default=defaultOptions['p12Certificate'], metavar='FILE')
        parser.add_option('--p12-password', dest='p12Password',
                          help='PKCS12 (P12) password. Default STRATUSLAB_P12_PASSWORD',
                          default=defaultOptions['p12Password'], metavar='PASSWORD')
        return parser

    @staticmethod
    def checkOptions(options):
        if options.p12Certificate and options.p12Password:
            return True

        if options.p12Certificate and not options.p12Password:
            prompt = "PKCS12 (P12) cert '%s' password: " % options.p12Certificate
            options.p12Password = getpass.getpass(prompt=prompt)
            return True

        return False


class CloudEndpoint(object):
    optionString = '--endpoint'

    @staticmethod
    def options():
        return {'endpoint': os.getenv('STRATUSLAB_ENDPOINT', '')}

    @staticmethod
    def addOptions(parser, defaultOptions=None):
        if not defaultOptions:
            defaultOptions = CloudEndpoint.options()

        parser.add_option('--endpoint', dest='endpoint',
                          help='cloud endpoint (hostname or URL). Default STRATUSLAB_ENDPOINT',
                          default=defaultOptions['endpoint'])

    @staticmethod
    def checkOptions(options):
        if options.endpoint:
            return True

        return False



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import re

import stratuslab.Exceptions as Exceptions
import stratuslab.Util as Util
from stratuslab.Compressor import Compressor
from stratuslab.marketplace.ManifestDownloader import ManifestDownloader
from stratuslab.ManifestInfo import ManifestInfo

class Image(object):

    re_imageId = re.compile('^[A-Za-z0-9_-]{27}$')
    re_diskId = re.compile('^[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}$')

    def __init__(self, configHolder):
        configHolder.assign(self)
        self.configHolder = configHolder

        self.manifestDownloader = self._createDownloader()

    def _createDownloader(self):
        configHolder = self.configHolder.copy()
        return ManifestDownloader(configHolder)
    
    def checkImageExists(self, imageId):
        """dereferences the entry and checks the image location if an image id"""
        if Image.isImageId(imageId):
            self._checkImageByIdInMarketplace(imageId)

    @staticmethod
    def checksumImage(filename, checksums=ManifestInfo.MANDATORY_CHECKSUMS):
        """Return dictionary of checksums."""

        return Compressor.checksum_file(filename, checksums)

    @staticmethod
    def isImageId(imageId):
        return Image.re_imageId.match(imageId)

    @staticmethod
    def isDiskId(diskId):
        return Image.re_diskId.match(diskId)

    def getImageFormatByImageId(self, imageId):
        return self._getImageElementValue('format', imageId)

    def getImageDisksBusTypeByImageId(self, imageId):
        return self._getImageElementValue('disks-bus', imageId)

    def getInboundPortsByImageId(self, imageId):
        return self._getImageElementValue('inboundports', imageId)

    def _getImageElementValue(self, element, imageId):
        if Image.isImageId(imageId):
            return self.manifestDownloader.getImageElementValue(element, imageId)
        else:
            raise Exceptions.ValidationException('Image ID was expected. Given %s' % imageId)
            
    def _checkImageByUrl(self, imageUrl):
        try:
            Util.checkUrlExists(imageUrl)
        except (Exceptions.ValidationException, Exceptions.ExecutionException), ex:
            raise Exceptions.ValidationException("Unable to access image '%s': %s" %
                                      (imageUrl, str(ex)))
        else:
            self.printDetail('Image available: %s' % imageUrl)

    def _checkImageByIdInMarketplace(self, imageId):
        imageLocations = self._getImageLocationsByImageId(imageId)
        if not imageLocations:
            raise Exceptions.ValidationException('Image location(s) are not set in manifest for the image with ID %s' % imageId)
        else:
            self.printDetail('Manifest with ID %s is available. Image locations: %s' % 
                             (imageId, str(imageLocations)))

    def _getImageLocationsByImageId(self, imageId):
        '''Return list of locations.'''
        self.manifestDownloader.downloadManifestByImageId(imageId)
        return self.manifestDownloader.getImageLocations()

    def printDetail(self, msg):
        return Util.printDetail(msg, self.verboseLevel, Util.VERBOSE_LEVEL_DETAILED)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import re
import copy

from ConfigParser import SafeConfigParser
import ConfigParser
import stratuslab.Util as Util
from stratuslab.Exceptions import ConfigurationException

class ConfigHolder(object):

    @staticmethod
    def configFileToDict(configFileName):
        config = ConfigHolder.parseConfig(configFileName)
        dict = ConfigHolder._convertToDict(config)
        return dict

    @staticmethod
    def configFileToDictWithFormattedKeys(configFileName, withMap=False):
        config = ConfigHolder.configFileToDict(configFileName)
        return ConfigHolder._formatConfigKeys(config, withMap)

    @staticmethod
    def configFileHandlerToDict(configFileHandler):
        config = SafeConfigParser()
        config.readfp(configFileHandler)
        dict = ConfigHolder._convertToDict(config)
        return dict

    @staticmethod
    def convertToSectionDict(config):
        dicts = {}
        for section in config.sections():
            dict = {}
            for k, v in config.items(section):
                dict[k] = v
            dicts[section] = dict
        return dicts

    @staticmethod
    def _convertToDict(config):
        dict = {}
        for section in config.sections():
            for k, v in config.items(section):
                _v = v
                if '\n' in v:
                    _v = ConfigHolder._convertToMultiLineValue(v)
                dict[k] = _v
        return dict

    @staticmethod
    def _convertToMultiLineValue(value):
        return ' ' + '\n '.join(filter(None, value.split('\n')))

    @staticmethod
    def parseConfig(configFileName):
        try:
            open(configFileName)
        except Exception as ex:
            msg = 'Cannot access configuration file %s: %s' % (configFileName,
                                                               str(ex))
            raise ConfigurationException(msg)
        config = SafeConfigParser()
        config.read(configFileName)
        return config

    @staticmethod
    def addConfigFileSysadminOption(parser):
        parser.add_option('-c', '--config', dest='configFile',
                            help='configuration file. Default %s' % Util.defaultConfigFile,
                            metavar='FILE',
                            default=Util.defaultConfigFile)
        return parser

    @staticmethod
    def addConfigFileUserOption(parser):
        parser.add_option('-c', '--config', dest='configFile',
                           help='user configuration file. Default %s' % Util.defaultConfigFileUser,
                           metavar='FILE',
                           default=Util.defaultConfigFileUser)
        parser.add_option('-S', '--user-config-section', dest='selected_section',
                           help='Section to load in the user configuration file. ' \
                           'Can also be set via environment variable: %s' % Util.userConfigFileSelectedSection,
                           default=os.getenv(Util.userConfigFileSelectedSection, None))
        return parser

    @staticmethod
    def _formatConfigKeys(config, withMap=False):
        _dict = {}
        _map = {}
        for k, v in config.items():
            camel = ConfigHolder._camelCase(k)
            _dict[camel] = v
            if withMap:
                _map[k] = camel
                _map[camel] = k
        if withMap:
            return _dict, _map
        else:
            return _dict

    @staticmethod
    def _camelCase(key):
        if '_' not in key:
            return key

        formattedKey = ''.join([part.title() for part in key.split('_')])
        if len(formattedKey) > 1:
            formattedKey = formattedKey[0].lower() + formattedKey[1:]
        return formattedKey

    def __init__(self, options={}, config={}):
        if not isinstance(options, dict):
            raise TypeError('options parameter must be omitted or a dictionary')
        if not isinstance(config, dict):
            raise TypeError('config parameter must be omitted or a dictionary')
        object.__setattr__(self, 'options', options)
        object.__setattr__(self, 'config', config)

    def assign(self, obj):
        Util.assignAttributes(obj, ConfigHolder._formatConfigKeys(self.config))
        Util.assignAttributes(obj, self.options)

    def copy(self):
        copy = ConfigHolder(self.options.copy(), self.config.copy())
        return copy

    def deepcopy(self):
        return self.__deepcopy__()

    def __deepcopy__(self, memo=dict()):
        deepCopy = ConfigHolder(copy.deepcopy(self.options), 
                                copy.deepcopy(self.config))
        return deepCopy

    def set(self, key, value):
        self.options[key] = value

    def __str__(self):
        output = '* %s:\n' % self.__class__.__name__
        for attr in ['options', 'config']:
            if getattr(self, attr):
                output += '** %s:\n' % attr.upper()
                pkeys = getattr(self, attr).keys()
                pkeys.sort()
                output += '\n'.join(['  %s = %s' % (k, getattr(self, attr)[k]) for k in pkeys]) + '\n'
        return output

    def __getattribute__(self, key):
        try:
            return object.__getattribute__(self, key)
        except AttributeError:
            pass
        try:
            return object.__getattribute__(self, 'options')[key]
        except KeyError:
            pass
        try:
            return object.__getattribute__(self, 'config')[key]
        except KeyError:
            pass
        raise AttributeError("'%s' object has no attribute '%s'" % (object.__getattribute__(self, '__class__'), key))

    def __setattr__(self, key, value):
        self.options[key] = value
    
    
class UserConfigurator(object):
    
    SELECTED_SECTION = 'selected_section'
    INSTANCE_TYPES_SECTION = 'instance_types'
    
    @staticmethod
    def configFileToDictWithFormattedKeys(configFile, withMap=False, selected_section=None):
        '''This accepts either a file-like object or a filename.'''
        usercfg = UserConfigurator(configFile)
        return UserConfigurator.userConfiguratorToDictWithFormattedKeys(usercfg, withMap, selected_section)

    @staticmethod
    def userConfiguratorToDictWithFormattedKeys(usercfg, withMap=False, selected_section=None):
        config = usercfg.getDict(selected_section)
        return ConfigHolder._formatConfigKeys(config, withMap)

    def __init__(self, configFile=Util.defaultConfigFileUser):
        '''Reads argument as file-like object first, then as a filename.
           Note that NO checks are made on the existence of the referenced file.'''
        self._parser = SafeConfigParser()

        try:
            try:
                self._parser.readfp(configFile) # file
            except AttributeError:
                self._parser.read(configFile) # filename
        except ConfigParser.ParsingError, ex:
            raise ConfigurationException(ex)

        self._initDict()

    def _initDict(self):
        self._dict = {}
        self._dict['userDefinedInstanceTypes'] = self.getUserDefinedInstanceTypes()
        
    def _loadDefaults(self):
        self._loadSection('default')
    
    def _loadSection(self, section):
        self._dict.update(dict(self._parser.items(section)))

    def getUserDefinedInstanceTypes(self):
        types = self.getSectionDict(UserConfigurator.INSTANCE_TYPES_SECTION)
        for name in types.keys():
            t = UserConfigurator._instanceTypeStringToTuple(types[name])
            if UserConfigurator._validInstanceTypeTuple(t):
                types[name] = t
            else:
                del types[name]
        return types

    @staticmethod
    def _validInstanceTypeTuple(t):
        if len(t) != 3:
            return None
        cpu, ram, swap = t
        if not isinstance(cpu, int) or cpu <= 0:
            return None
        if not isinstance(ram, int) or ram <=0:
            return None
        if not isinstance(swap, int) or swap < 0:
            return None
        return t

    @staticmethod
    def _instanceTypeStringToTuple(s):
        return tuple(map (int, re.findall('\d+', s)))

    def getDefaultInstanceType(self):
        if (hasattr(self, 'defaultInstanceType') and self.defaultInstanceType):
            return self.defaultInstanceType
        else:
            return None

    def getDict(self, selected_section=None):
        self._initDict()

        try:
            self._loadDefaults()
        except ConfigParser.NoSectionError, ex:
            raise ConfigurationException(ex)

        if not selected_section:
            if self._parser.has_option('default', UserConfigurator.SELECTED_SECTION):
                selected_section = self._parser.get('default', UserConfigurator.SELECTED_SECTION)

        if selected_section:
            try:
                self._loadSection(selected_section)
            except ConfigParser.NoSectionError, ex:
                raise ConfigurationException(ex)

        ConfigHolder._formatConfigKeys(self._dict)

        return self._dict

    def getSectionDict(self, section=None):
        values = {}

        if section:
            try:
                values = dict(self._parser.items(section))
            except ConfigParser.NoSectionError:
                pass

        return values



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from stratuslab.installator.OpenNebulaCommon import OpenNebulaCommon
from stratuslab.CloudConnectorFactory import CloudConnectorFactory
from stratuslab.Monitor import Monitor
from stratuslab.Exceptions import InputException
from stratuslab.Authn import AuthnFactory

class Registrar(OpenNebulaCommon):

    def __init__(self, configHolder):
        self.configHolder = configHolder
        self.config = configHolder.config
        self.deRegister = False
        configHolder.assign(self)
        self.username = 'oneadmin'
        
        credentials = AuthnFactory.getCredentials(self)
        self.cloud = CloudConnectorFactory.getCloud(credentials)
        self.cloud.setEndpointFromParts(self.frontendIp,
                                        self.proxyPort)        

        self._assignDrivers()

    def register(self, hostnames):
        
        id = None
        for hostname in hostnames:
            if self.deRegister:
                self.deregister(hostname)
            else:
                id = self._register(hostname)
        return id

    def deregister(self, hostname):
        try:
            id = int(hostname)
        except:
            id = self._getHostnameId(hostname)
        self._removeCloudNode(id)

    def _getHostnameId(self, hostname):
        monitor = Monitor(self.configHolder)
        infoList = monitor.listNodes()
        info = None
        for i in infoList:
            if i.name == hostname:
                info = i
                break
        if not info:
            raise InputException('Failed to find node with hostname %s' % hostname)
        return int(info.id)
        

    def _register(self, hostname):
        
        self.nodeAddr = hostname
        return self._addCloudNode()
        



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552."
#
# Copyright (c) 2011, SixSq Sarl
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import socket
import sys
import os
from optparse import OptionParser
import xmlrpclib
import json
import logging

import stratuslab.api.LogUtil as LogUtil

import stratuslab.Util as Util
from stratuslab.VersionChecker import VersionChecker
import stratuslab.Exceptions as Exceptions
from stratuslab.ConfigHolder import ConfigHolder, UserConfigurator
from stratuslab.Exceptions import ConfigurationException


class CommandBase(object):
    def __init__(self):
        self.options = None
        self.config = {}
        self._configKeysClassAttrsTwoWayMap = {}

        self.verboseLevel = 0
        self.quiet = 0
        self.parser = None
        self._setParserAndParse()

        try:
            self, self._loadConfigFileAndUpdateOptions()
        except Exception, ex:
            self.raiseOrDisplayError(str(ex))

        self.checkOptions()
        self._callAndHandleErrors(self, self.doWork.__name__)

    def _getLoggingLevel(self):
        if self.quiet >= 3:
            return logging.CRITICAL
        elif self.quiet == 2:
            return logging.ERROR
        elif self.quiet == 1:
            return logging.WARNING
        elif self.verboseLevel == 0:
            return logging.INFO
        else:
            return logging.DEBUG

    def _setParserAndParse(self):
        self.parser = OptionParser(version="13.05.0.RC1")

        self.parser.add_option('-v', '--verbose', dest='verboseLevel',
                               help='verbose level (use multiple times for more detail)',
                               action='count', default=self.verboseLevel)

        self.parser.add_option('-q', '--quiet', dest='quiet',
                               help='quiet output (use multiple times for less detail)',
                               action='count', default=self.quiet)

        self._addConfigFileOption()

        self.parse()
        self.verboseLevel = self.options.verboseLevel
        self.quiet = self.options.quiet

        LogUtil.set_logger_level(level=self._getLoggingLevel())

    def _addConfigFileOption(self):
        pass

    def _loadConfigFileAndUpdateOptions(self):
        pass

    def _callAndHandleErrors(self, methodName, *args, **kw):

        try:
            Util.runMethodByName(methodName, *args, **kw)
        except ValueError, ex:
            self.raiseOrDisplayError('Error: %s\n' % str(ex))
        except xmlrpclib.ProtocolError, ex:
            self.raiseOrDisplayError('Error: %s' % ex.errmsg)
        except socket.sslerror, ex:
            self._checkPythonVersionAndRaise()
            raise ex
            #self.raiseOrDisplayError('Network error: %s' % ex)
        except socket.error, ex:
            self.raiseOrDisplayError('Network error: %s' % ex)
        except Exceptions.ClientException, ex:
            msg = 'Error: ' + ex.reason
            if ex.content and (self.verboseLevel > 1):
                msg += '\nDetail: ' + ex.content
            if getattr(ex, 'mediaType', None) == 'json':
                error = json.loads(ex.content)
                msg = 'Error: %s (code %d)' % (error['message'], error['code'])
            self.raiseOrDisplayError(msg)
        except Exception, ex:
            self.raiseOrDisplayError('Error: %s' % ex)

    def _checkPythonVersionAndRaise(self):
        try:
            VersionChecker().check()
        except Exceptions.ValidationException, ex:
            self.raiseOrDisplayError(ex)

    def parse(self):
        pass

    def checkOptions(self):
        pass

    def checkArgumentsLength(self):
        pass

    def usageExitTooFewArguments(self):
        return self.parser.error('Too few arguments')

    def usageExitTooManyArguments(self):
        return self.parser.error('Too many arguments')

    def usageExitWrongNumberOfArguments(self):
        return self.parser.error('Wrong number of arguments')

    def raiseOrDisplayError(self, errorMsg):
        if self.verboseLevel > 2:
            raise
        else:
            Util.printError(errorMsg, exit=False)
        sys.exit(-1)

    def printDetail(self, message):
        Util.printDetail(message, self)

    def printError(self, message):
        Util.printError(message, self)

    def printMandatoryOptionError(self, option):
        self.printError('Missing mandatory %s option' % option)


class CommandBaseUser(CommandBase):
    def _addConfigFileOption(self):
        ConfigHolder.addConfigFileUserOption(self.parser)

    def _loadConfigFileAndUpdateOptions(self):
        self._loadConfigFile()
        self._updateOptionsFromConfigFile()

    def _loadConfigFile(self):
        if not hasattr(self.options, 'configFile'):
            return

        configFile = self.options.configFile

        if configFile == Util.defaultConfigFileUser:
            if not os.path.exists(configFile):
                Util.printDetail('[WARNING] Default user configuration file does not exist: %s' %
                                 configFile, verboseLevel=self.verboseLevel)
                return

        selected_section = None
        if hasattr(self.options, 'selected_section'):
            selected_section = self.options.selected_section

        try:
            self.config, self._configKeysClassAttrsTwoWayMap = \
                UserConfigurator.configFileToDictWithFormattedKeys(configFile, withMap=True,
                                                                   selected_section=selected_section)
        except ConfigurationException, ex:
            raise ConfigurationException('Error parsing user configuration file %s' % configFile + '. Details: %s' % ex)

    def _updateOptionsFromConfigFile(self):
        """
        Order of precedence:
        * command line option
        * environment variable
        * configuration file
        * default value

        Update of the corresponding options object key/value pairs can only be
        done if neither corresponding command line option nor environment
        variable was given/set.

        The following key/attribute/env.var. naming is assumed:

         configuration file | class attribute | environment variable
         ----------------- + --------------- + --------------------
         p12_certificate   | p12Certificate  | STRATUSLAB_P12_CERTIFICATE
        """

        if not self.config:
            return

        for k in self.config:
            if k in self.options.__dict__:
                valueFromOptions = getattr(self.options, k)
                # Set the value from configuration file:
                # * if attribute is empty
                if not valueFromOptions:
                    setattr(self.options, k, self.config[k])
                # * if default is equal to "provided" value, we may assume that
                #   the option wasn't given on command line; but we are going
                #   to double check this in each Option object of the parser as
                #   the same value might have been provided via environment var.
                elif valueFromOptions == self.parser.defaults[k]:
                    for optionObj in self.parser.option_list:
                        # work only with Option object for the particular key
                        if optionObj.dest != k:
                            continue
                            # Update if
                        # * the long option is NOT in the list of the CLI arguments
                        if not (optionObj._long_opts[0] in sys.argv):
                            # * and not set via corresponding environment variable
                            envVar = 'STRATUSLAB_%s' % self._configKeysClassAttrsTwoWayMap[k].upper()
                            if not os.getenv(envVar):
                                setattr(self.options, k, self.config[k])
                else:
                    pass


class CommandBaseSysadmin(CommandBase):
    def _addConfigFileOption(self):
        ConfigHolder.addConfigFileSysadminOption(self.parser)



#
# Created as part of the StratusLab project (http://stratuslab.eu),
# co-funded by the European Commission under the Grant Agreement
# INFSO-RI-261552.
#
# Copyright (c) 2012, Centre National de la Recherche Scientifique
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import cgi
import json
import os
import sys

from stratuslab.ConfigHolder import ConfigHolder
from stratuslab.Exceptions import ConfigurationException
from stratuslab.pat.Client import PortTranslationDbClient

# FIXME: Is this class actually used anywhere?
class PortTranslationService(object):
    def __init__(self, configFile=''):
        self.configFile = configFile
        configHolder = ConfigHolder(config=self._loadConfiguration())
        self.portTranslation = PortTranslationDbClient(configHolder)

    def _loadConfiguration(self):
        return ConfigHolder.configFileToDict(self._findConfigFile())

    def _findConfigFile(self):
        cgiConfig = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])), 'conf/stratuslab.cfg')
        paths = (self.configFile, cgiConfig)
        for path in paths:
            if os.path.exists(path):
                return path
        raise(ConfigurationException('Missing configuration file'))

    def _sendJson(self, data):
        jsonData = json.dumps(data)
        print "Content-type: application/json\r\n\r\n"
        print jsonData

    def _convertPortTranslationToJson(self, cleanPorts):
        rawPorts = {}
        for vmId, vmPorts in cleanPorts.iteritems():
            rawPorts[vmId] = vmPorts.toList()
        return rawPorts

    def run(self):
        form = cgi.FieldStorage()
        vmId = form.getvalue('vmid')
        cleanPorts = self.portTranslation.getPortTranslation(vmId)
        rawPorts = self._convertPortTranslationToJson(cleanPorts)
        self._sendJson(rawPorts)

